@@ -1,9 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *           Michael Zucchi <notzed@ximian.com>
- *
- *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -347,7 +343,7 @@ g_mime_charset_language (const char *charset)
 		return NULL;
 	
 	for (i = 0; i < G_N_ELEMENTS (cjkr_lang_map); i++) {
-		if (!strcasecmp (cjkr_lang_map[i].charset, charset))
+		if (!g_ascii_strcasecmp (cjkr_lang_map[i].charset, charset))
 			return cjkr_lang_map[i].lang;
 	}
 	
@@ -521,7 +517,7 @@ g_mime_charset_canon_name (const char *charset)
 		return NULL;
 	
 	charset = g_mime_charset_iconv_name (charset);
-	if (strncasecmp (charset, "iso", 3) == 0) {
+	if (g_ascii_strncasecmp (charset, "iso", 3) == 0) {
 		ptr = charset + 3;
 		if (*ptr == '-' || *ptr == '_')
 			ptr++;
@@ -624,23 +620,23 @@ g_mime_charset_iso_to_windows (const char *isocharset)
 	
 	isocharset = g_mime_charset_canon_name (isocharset);
 	
-	if (!strcasecmp (isocharset, "iso-8859-1") || !strcasecmp (isocharset, "us-ascii"))
+	if (!g_ascii_strcasecmp (isocharset, "iso-8859-1") || !g_ascii_strcasecmp (isocharset, "us-ascii"))
 		return "windows-cp1252";
-	else if (!strcasecmp (isocharset, "iso-8859-2"))
+	else if (!g_ascii_strcasecmp (isocharset, "iso-8859-2"))
 		return "windows-cp1250";
-	else if (!strcasecmp (isocharset, "iso-8859-4"))
+	else if (!g_ascii_strcasecmp (isocharset, "iso-8859-4"))
 		return "windows-cp1257";
-	else if (!strcasecmp (isocharset, "iso-8859-5"))
+	else if (!g_ascii_strcasecmp (isocharset, "iso-8859-5"))
 		return "windows-cp1251";
-	else if (!strcasecmp (isocharset, "iso-8859-6"))
+	else if (!g_ascii_strcasecmp (isocharset, "iso-8859-6"))
 		return "windows-cp1256";
-	else if (!strcasecmp (isocharset, "iso-8859-7"))
+	else if (!g_ascii_strcasecmp (isocharset, "iso-8859-7"))
 		return "windows-cp1253";
-	else if (!strcasecmp (isocharset, "iso-8859-8"))
+	else if (!g_ascii_strcasecmp (isocharset, "iso-8859-8"))
 		return "windows-cp1255";
-	else if (!strcasecmp (isocharset, "iso-8859-9"))
+	else if (!g_ascii_strcasecmp (isocharset, "iso-8859-9"))
 		return "windows-cp1254";
-	else if (!strcasecmp (isocharset, "iso-8859-13"))
+	else if (!g_ascii_strcasecmp (isocharset, "iso-8859-13"))
 		return "windows-cp1257";
 	
 	return isocharset;
