@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,21 +15,17 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
 #ifndef __GMIME_DISPOSITION_H__
 #define __GMIME_DISPOSITION_H__
 
-#ifdef __cplusplus
-extern "C" {
-#pragma }
-#endif /* __cplusplus */
-
 #include <glib.h>
 #include <gmime/gmime-param.h>
 
+G_BEGIN_DECLS
+
 
 /**
  * GMIME_DISPOSITION_ATTACHMENT:
@@ -70,8 +64,6 @@ const char *g_mime_disposition_get_parameter (GMimeDisposition *disposition, con
 
 char *g_mime_disposition_header (GMimeDisposition *disposition, gboolean fold);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __GMIME_DISPOSITION_H__ */
