@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2003-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -187,7 +184,7 @@ enriched_tag_needs_param (const char *tag)
 	int i;
 	
 	for (i = 0; i < NUM_ENRICHED_TAGS; i++)
-		if (!strcasecmp (tag, enriched_tags[i].enriched))
+		if (!g_ascii_strcasecmp (tag, enriched_tags[i].enriched))
 			return enriched_tags[i].needs_param;
 	
 	return FALSE;
@@ -215,7 +212,7 @@ param_parse_colour (const char *inptr, int inlen)
 	int i;
 	
 	for (i = 0; i < NUM_VALID_COLOURS; i++) {
-		if (!strncasecmp (inptr, valid_colours[i], inlen))
+		if (!g_ascii_strncasecmp (inptr, valid_colours[i], inlen))
 			return g_strdup (valid_colours[i]);
 	}
 	
@@ -283,7 +280,7 @@ param_parse (const char *enriched, const char *inptr, int inlen)
 	int i;
 	
 	for (i = 0; i < NUM_ENRICHED_TAGS; i++) {
-		if (!strcasecmp (enriched, enriched_tags[i].enriched))
+		if (!g_ascii_strcasecmp (enriched, enriched_tags[i].enriched))
 			return enriched_tags[i].parse_param (inptr, inlen);
 	}
 	
@@ -334,12 +331,29 @@ enriched_to_html (GMimeFilter *filter, char *in, size_t inlen, size_t prespace,
 			
 			break;
 		case '\n':
-			if (!(enriched->flags & IS_RICHTEXT) && enriched->nofill <= 0) {
+			if (!(enriched->flags & IS_RICHTEXT)) {
 				/* text/enriched */
-				while (inptr < inend && (outptr + 4) < outend && *inptr == '\n') {
-					memcpy (outptr, "<br>", 4);
-					outptr += 4;
-					inptr++;
+				if (enriched->nofill > 0) {
+					if ((outptr + 4) < outend) {
+						memcpy (outptr, "<br>", 4);
+						outptr += 4;
+					} else {
+						inptr--;
+						goto backup;
+					}
+				} else if (*inptr == '\n') {
+					if ((outptr + 4) >= outend) {
+						inptr--;
+						goto backup;
+					}
+					
+					while (inptr < inend && (outptr + 4) < outend && *inptr == '\n') {
+						memcpy (outptr, "<br>", 4);
+						outptr += 4;
+						inptr++;
+					}
+				} else {
+					*outptr++ = ' ';
 				}
 			} else {
 				/* text/richtext */
@@ -407,20 +421,16 @@ enriched_to_html (GMimeFilter *filter, char *in, size_t inlen, size_t prespace,
 				goto need_input;
 			}
 			
-			if (!strncasecmp (tag, "nofill>", 7)) {
+			if (!g_ascii_strncasecmp (tag, "nofill>", 7)) {
 				if ((outptr + 5) < outend) {
-					memcpy (outptr, "<pre>", 5);
 					enriched->nofill++;
-					outptr += 5;
 				} else {
 					inptr = tag - 1;
 					goto backup;
 				}
-			} else if (!strncasecmp (tag, "/nofill>", 8)) {
+			} else if (!g_ascii_strncasecmp (tag, "/nofill>", 8)) {
 				if ((outptr + 6) < outend) {
-					memcpy (outptr, "</pre>", 6);
 					enriched->nofill--;
-					outptr += 6;
 				} else {
 					inptr = tag - 1;
 					goto backup;
@@ -451,7 +461,7 @@ enriched_to_html (GMimeFilter *filter, char *in, size_t inlen, size_t prespace,
 							goto need_input;
 						}
 						
-						if (strncasecmp (inptr, "<param>", 7) != 0) {
+						if (g_ascii_strncasecmp (inptr, "<param>", 7) != 0) {
 							/* ignore the enriched command tag... */
 							inptr -= 1;
 							goto loop;
@@ -468,7 +478,7 @@ enriched_to_html (GMimeFilter *filter, char *in, size_t inlen, size_t prespace,
 							goto need_input;
 						}
 						
-						if (strncasecmp (inptr, "</param>", 8) != 0) {
+						if (g_ascii_strncasecmp (inptr, "</param>", 8) != 0) {
 							/* ignore the enriched command tag... */
 							inptr += 7;
 							goto loop;
