@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2003-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,26 +15,22 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
 #ifndef __GMIME_FILTER_ENRICHED_H__
 #define __GMIME_FILTER_ENRICHED_H__
 
-#ifdef __cplusplus
-extern "C" {
-#pragma }
-#endif /* __cplusplus */
-
 #include <gmime/gmime-filter.h>
 
+G_BEGIN_DECLS
+
 #define GMIME_TYPE_FILTER_ENRICHED            (g_mime_filter_enriched_get_type ())
-#define GMIME_FILTER_ENRICHED(obj)            (GMIME_CHECK_CAST ((obj), GMIME_TYPE_FILTER_ENRICHED, GMimeFilterEnriched))
-#define GMIME_FILTER_ENRICHED_CLASS(klass)    (GMIME_CHECK_CLASS_CAST ((klass), GMIME_TYPE_FILTER_ENRICHED, GMimeFilterEnrichedClass))
-#define GMIME_IS_FILTER_ENRICHED(obj)         (GMIME_CHECK_TYPE ((obj), GMIME_TYPE_FILTER_ENRICHED))
-#define GMIME_IS_FILTER_ENRICHED_CLASS(klass) (GMIME_CHECK_CLASS_TYPE ((klass), GMIME_TYPE_FILTER_ENRICHED))
-#define GMIME_FILTER_ENRICHED_GET_CLASS(obj)  (GMIME_CHECK_GET_CLASS ((obj), GMIME_TYPE_FILTER_ENRICHED, GMimeFilterEnrichedClass))
+#define GMIME_FILTER_ENRICHED(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GMIME_TYPE_FILTER_ENRICHED, GMimeFilterEnriched))
+#define GMIME_FILTER_ENRICHED_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GMIME_TYPE_FILTER_ENRICHED, GMimeFilterEnrichedClass))
+#define GMIME_IS_FILTER_ENRICHED(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GMIME_TYPE_FILTER_ENRICHED))
+#define GMIME_IS_FILTER_ENRICHED_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GMIME_TYPE_FILTER_ENRICHED))
+#define GMIME_FILTER_ENRICHED_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GMIME_TYPE_FILTER_ENRICHED, GMimeFilterEnrichedClass))
 
 typedef struct _GMimeFilterEnriched GMimeFilterEnriched;
 typedef struct _GMimeFilterEnrichedClass GMimeFilterEnrichedClass;
@@ -60,8 +54,6 @@ GType g_mime_filter_enriched_get_type (void);
 
 GMimeFilter *g_mime_filter_enriched_new (guint32 flags);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __GMIME_FILTER_ENRICHED_H__ */
