@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -48,10 +45,13 @@
 #include "gmime-charset.h"
 #include "gmime-error.h"
 
+#ifdef ENABLE_DEBUG
 #define d(x) x
-#define _(x) x
+#else
+#define d(x)
+#endif
 
-#define GMIME_ERROR_QUARK (g_quark_from_static_string ("gmime"))
+#define _(x) x
 
 static void g_mime_gpg_context_class_init (GMimeGpgContextClass *klass);
 static void g_mime_gpg_context_init (GMimeGpgContext *ctx, GMimeGpgContextClass *klass);
@@ -161,20 +161,20 @@ gpg_hash_id (GMimeCipherContext *ctx, const char *hash)
 	if (hash == NULL)
 		return GMIME_CIPHER_HASH_DEFAULT;
 	
-	if (!strcasecmp (hash, "pgp-"))
+	if (!g_ascii_strcasecmp (hash, "pgp-"))
 		hash += 4;
 	
-	if (!strcasecmp (hash, "md2"))
+	if (!g_ascii_strcasecmp (hash, "md2"))
 		return GMIME_CIPHER_HASH_MD2;
-	else if (!strcasecmp (hash, "md5"))
+	else if (!g_ascii_strcasecmp (hash, "md5"))
 		return GMIME_CIPHER_HASH_MD5;
-	else if (!strcasecmp (hash, "sha1"))
+	else if (!g_ascii_strcasecmp (hash, "sha1"))
 		return GMIME_CIPHER_HASH_SHA1;
-	else if (!strcasecmp (hash, "ripemd160"))
+	else if (!g_ascii_strcasecmp (hash, "ripemd160"))
 		return GMIME_CIPHER_HASH_RIPEMD160;
-	else if (!strcasecmp (hash, "tiger192"))
+	else if (!g_ascii_strcasecmp (hash, "tiger192"))
 		return GMIME_CIPHER_HASH_TIGER192;
-	else if (!strcasecmp (hash, "haval-5-160"))
+	else if (!g_ascii_strcasecmp (hash, "haval-5-160"))
 		return GMIME_CIPHER_HASH_HAVAL5160;
 	
 	return GMIME_CIPHER_HASH_DEFAULT;
@@ -197,7 +197,7 @@ gpg_hash_name (GMimeCipherContext *ctx, GMimeCipherHash hash)
 	case GMIME_CIPHER_HASH_HAVAL5160:
 		return "pgp-haval-5-160";
 	default:
-		return NULL;
+		return "pgp-sha1";
 	}
 }
 
@@ -211,14 +211,6 @@ enum _GpgCtxMode {
 	GPG_CTX_MODE_EXPORT,
 };
 
-enum _GpgTrustMetric {
-	GPG_TRUST_UNKNOWN,
-	GPG_TRUST_NEVER,
-	GPG_TRUST_MARGINAL,
-	GPG_TRUST_FULLY,
-	GPG_TRUST_ULTIMATE
-};
-
 struct _GpgCtx {
 	enum _GpgCtxMode mode;
 	GMimeSession *session;
@@ -242,6 +234,7 @@ struct _GpgCtx {
 	unsigned char *statusptr;
 	unsigned int statusleft;
 	
+	char *need_id;
 	char *passwd;
 	
 	GMimeStream *istream;
@@ -250,8 +243,12 @@ struct _GpgCtx {
 	GByteArray *diag;
 	GMimeStream *diagnostics;
 	
+	GMimeSigner *signers;
+	GMimeSigner *signer;
+	
 	int exit_status;
 	
+	unsigned int utf8:1;
 	unsigned int exited:1;
 	unsigned int complete:1;
 	unsigned int seen_eof1:1;
@@ -264,10 +261,14 @@ struct _GpgCtx {
 	
 	unsigned int bad_passwds:2;
 	
+	unsigned int badsig:1;
+	unsigned int errsig:1;
+	unsigned int goodsig:1;
 	unsigned int validsig:1;
-	unsigned int trust:3;
+	unsigned int nopubkey:1;
+	unsigned int nodata:1;
 	
-	unsigned int padding:17;
+	unsigned int padding:14;
 };
 
 static struct _GpgCtx *
@@ -310,31 +311,42 @@ gpg_ctx_new (GMimeSession *session, const char *path)
 	gpg->bad_passwds = 0;
 	gpg->need_passwd = FALSE;
 	gpg->send_passwd = FALSE;
+	gpg->need_id = NULL;
 	gpg->passwd = NULL;
 	
+	gpg->badsig = FALSE;
+	gpg->errsig = FALSE;
+	gpg->goodsig = FALSE;
 	gpg->validsig = FALSE;
-	gpg->trust = GPG_TRUST_UNKNOWN;
+	gpg->nopubkey = FALSE;
+	
+	gpg->signers = NULL;
+	gpg->signer = (GMimeSigner *) &gpg->signers;
 	
 	gpg->istream = NULL;
 	gpg->ostream = NULL;
 	
 	stream = g_mime_stream_mem_new ();
 	gpg->diag = GMIME_STREAM_MEM (stream)->buffer;
-	charset = g_mime_charset_locale_name ();
-	if (strcasecmp (charset, "UTF-8") != 0) {
+	charset = g_mime_locale_charset ();
+	if (g_ascii_strcasecmp (charset, "UTF-8") != 0) {
 		GMimeStream *fstream;
 		GMimeFilter *filter;
 		
 		filter = g_mime_filter_charset_new (charset, "UTF-8");
 		fstream = g_mime_stream_filter_new_with_stream (stream);
 		g_mime_stream_filter_add ((GMimeStreamFilter *) fstream, filter);
-		g_mime_stream_unref (stream);
+		g_object_unref (stream);
 		g_object_unref (filter);
 		
 		gpg->diagnostics = fstream;
+		
+		gpg->utf8 = FALSE;
 	} else {
 		/* system charset is UTF-8, shouldn't need any conversion */
 		gpg->diagnostics = stream;
+		
+		gpg->utf8 = TRUE;
 	}
 	
 	return gpg;
@@ -394,18 +406,18 @@ gpg_ctx_set_armor (struct _GpgCtx *gpg, gboolean armor)
 static void
 gpg_ctx_set_istream (struct _GpgCtx *gpg, GMimeStream *istream)
 {
-	g_mime_stream_ref (istream);
+	g_object_ref (istream);
 	if (gpg->istream)
-		g_mime_stream_unref (gpg->istream);
+		g_object_unref (gpg->istream);
 	gpg->istream = istream;
 }
 
 static void
 gpg_ctx_set_ostream (struct _GpgCtx *gpg, GMimeStream *ostream)
 {
-	g_mime_stream_ref (ostream);
+	g_object_ref (ostream);
 	if (gpg->ostream)
-		g_mime_stream_unref (gpg->ostream);
+		g_object_unref (gpg->ostream);
 	gpg->ostream = ostream;
 	gpg->seen_eof1 = FALSE;
 }
@@ -432,6 +444,7 @@ userid_hint_free (gpointer key, gpointer value, gpointer user_data)
 static void
 gpg_ctx_free (struct _GpgCtx *gpg)
 {
+	GMimeSigner *signer, *next;
 	int i;
 	
 	if (gpg->session)
@@ -466,16 +479,27 @@ gpg_ctx_free (struct _GpgCtx *gpg)
 	
 	g_free (gpg->statusbuf);
 	
-	if (gpg->passwd)
+	g_free (gpg->need_id);
+	
+	if (gpg->passwd) {
+		memset (gpg->passwd, 0, strlen (gpg->passwd));
 		g_free (gpg->passwd);
+	}
 	
 	if (gpg->istream)
-		g_mime_stream_unref (gpg->istream);
+		g_object_unref (gpg->istream);
 	
 	if (gpg->ostream)
-		g_mime_stream_unref (gpg->ostream);
+		g_object_unref (gpg->ostream);
 	
-	g_mime_stream_unref (gpg->diagnostics);
+	g_object_unref (gpg->diagnostics);
+	
+	signer = gpg->signers;
+	while (signer != NULL) {
+		next = signer->next;
+		g_mime_signer_free (signer);
+		signer = next;
+	}
 	
 	g_free (gpg);
 }
@@ -640,9 +664,8 @@ gpg_ctx_op_start (struct _GpgCtx *gpg)
 		setsid ();
 		
 		maxfd = sysconf (_SC_OPEN_MAX);
-		/* Loop over all fds. */
 		for (i = 3; i < maxfd; i++) {
-			/* don't close status-fd or passwd-fd */
+			/* don't close the status-fd or the passwd-fd */
 			if (i != fds[7] && i != fds[8])
 				fcntl (i, F_SETFD, FD_CLOEXEC);
 		}
@@ -734,6 +757,8 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 {
 	register unsigned char *inptr;
 	const unsigned char *status;
+	size_t nread, nwritten;
+	GMimeSigner *signer;
 	int len;
 	
  parse:
@@ -753,7 +778,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 	d(printf ("status: %s\n", status));
 	
 	if (strncmp (status, "[GNUPG:] ", 9) != 0) {
-		g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_PARSE_ERROR,
+		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
 			     _("Unexpected GnuPG status message encountered:\n\n%s"),
 			     status);
 		return -1;
@@ -766,9 +791,10 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 		char *hint, *user;
 		
 		status += 12;
+		
 		status = next_token (status, &hint);
 		if (!hint) {
-			g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_PARSE_ERROR,
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
 				     _("Failed to parse gpg userid hint."));
 			return -1;
 		}
@@ -779,50 +805,85 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 			goto recycle;
 		}
 		
-		if (!(user = g_locale_to_utf8 (status, -1, &nread, &nwritten, NULL)))
+		if (gpg->utf8 || !(user = g_locale_to_utf8 (status, -1, &nread, &nwritten, NULL)))
 			user = g_strdup (status);
 		
 		g_strstrip (user);
 		
 		g_hash_table_insert (gpg->userid_hint, hint, user);
 	} else if (!strncmp (status, "NEED_PASSPHRASE ", 16)) {
-		char *prompt, *userid, *passwd;
-		const char *name;
+		char *userid;
 		
 		status += 16;
 		
 		status = next_token (status, &userid);
 		if (!userid) {
-			g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_PARSE_ERROR,
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
 				     _("Failed to parse gpg passphrase request."));
 			return -1;
 		}
 		
-		name = g_hash_table_lookup (gpg->userid_hint, userid);
-		if (!name)
-			name = userid;
+		g_free (gpg->need_id);
+		gpg->need_id = userid;
+	} else if (!strncmp (status, "NEED_PASSPHRASE_PIN ", 20)) {
+		char *userid;
 		
-		prompt = g_strdup_printf (_("You need a passphrase to unlock the key for\n"
-					    "user: \"%s\""), name);
+		status += 20;
 		
-		passwd = g_mime_session_request_passwd (gpg->session, prompt, TRUE, userid, err);
-		g_free (prompt);
-		
-		g_free (gpg->userid);
-		gpg->userid = userid;
-		
-		if (passwd == NULL) {
-			if (err && *err == NULL)
-				g_set_error (err, GMIME_ERROR_QUARK, ECANCELED, _("Canceled."));
-			
+		status = next_token (status, &userid);
+		if (!userid) {
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
+				     _("Failed to parse gpg passphrase request."));
 			return -1;
 		}
 		
-		gpg->passwd = g_strdup_printf ("%s\n", passwd);
-		memset (passwd, 0, strlen (passwd));
-		g_free (passwd);
+		g_free (gpg->need_id);
+		gpg->need_id = userid;
+	} else if (!strncmp (status, "GET_HIDDEN ", 11)) {
+		char *prompt = NULL;
+		char *passwd = NULL;
+		const char *name;
 		
-		gpg->send_passwd = TRUE;
+		status += 11;
+		
+		if (!(name = g_hash_table_lookup (gpg->userid_hint, gpg->need_id)))
+			name = gpg->userid;
+		
+		if (!strncmp (status, "passphrase.pin.ask", 18)) {
+			prompt = g_strdup_printf (_("You need a PIN to unlock the key for your\n"
+						    "SmartCard: \"%s\""), name);
+		} else if (!strncmp (status, "passphrase.enter", 16)) {
+			prompt = g_strdup_printf (_("You need a passphrase to unlock the key for\n"
+						    "user: \"%s\""), name);
+		} else {
+			next_token (status, &prompt);
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_GENERAL,
+				     _("Unexpected request from GnuPG for `%s'"), prompt);
+			g_free (prompt);
+			return -1;
+		}
+		
+		if ((passwd = g_mime_session_request_passwd (gpg->session, prompt, TRUE, gpg->need_id, err))) {
+			if (!gpg->utf8) {
+				char *locale_passwd;
+				
+				if ((locale_passwd = g_locale_from_utf8 (passwd, -1, &nread, &nwritten, NULL))) {
+					memset (passwd, 0, strlen (passwd));
+					g_free (passwd);
+					passwd = locale_passwd;
+				}
+			}
+			
+			gpg->passwd = g_strdup_printf ("%s\n", passwd);
+			memset (passwd, 0, strlen (passwd));
+			g_free (passwd);
+			
+			gpg->send_passwd = TRUE;
+		} else if (err && *err == NULL) {
+			g_set_error (err, GMIME_ERROR, ECANCELED, _("Canceled."));
+		}
+		
+		g_free (prompt);
 	} else if (!strncmp (status, "GOOD_PASSPHRASE", 15)) {
 		gpg->bad_passwds = 0;
 	} else if (!strncmp (status, "BAD_PASSPHRASE", 14)) {
@@ -831,13 +892,13 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 		g_mime_session_forget_passwd (gpg->session, gpg->userid, err);
 		
 		if (gpg->bad_passwds == 3) {
-			g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_BAD_PASSWORD,
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_BAD_PASSWORD,
 				     _("Failed to unlock secret key: 3 bad passphrases given."));
 			return -1;
 		}
 	} else if (!strncmp (status, "UNEXPECTED ", 11)) {
 		/* this is an error */
-		g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_GENERAL,
+		g_set_error (err, GMIME_ERROR, GMIME_ERROR_GENERAL,
 			     _("Unexpected response from GnuPG: %s"),
 			     status + 11);
 		return -1;
@@ -847,12 +908,14 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 		
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		if (diagnostics && *diagnostics)
-			g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_GENERAL,
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_GENERAL,
 				     diagnostics);
 		else
-			g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_GENERAL,
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_GENERAL,
 				     _("No data provided"));
 		
+		gpg->nodata = TRUE;
+		
 		return -1;
 	} else {
 		switch (gpg->mode) {
@@ -862,24 +925,114 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 			}
 			break;
 		case GPG_CTX_MODE_VERIFY:
-			if (!strncmp (status, "TRUST_", 6)) {
-				status += 6;
-				if (!strncmp (status, "NEVER", 5)) {
-					gpg->trust = GPG_TRUST_NEVER;
-				} else if (!strncmp (status, "MARGINAL", 8)) {
-					gpg->trust = GPG_TRUST_MARGINAL;
-				} else if (!strncmp (status, "FULLY", 5)) {
-					gpg->trust = GPG_TRUST_FULLY;
-				} else if (!strncmp (status, "ULTIMATE", 8)) {
-					gpg->trust = GPG_TRUST_ULTIMATE;
-				}
-			} else if (!strncmp (status, "VALIDSIG", 8)) {
+			if (!strncmp (status, "SIG_ID ", 7)) {
+				/* not sure if this contains anything we care about... */
+			} else if (!strncmp (status, "GOODSIG ", 8)) {
+				gpg->goodsig = TRUE;
+				status += 8;
+				
+				signer = g_mime_signer_new ();
+				signer->status = GMIME_SIGNER_STATUS_GOOD;
+				gpg->signer->next = signer;
+				gpg->signer = signer;
+				
+				/* get the key id of the signer */
+				status = next_token (status, &signer->keyid);
+				
+				/* the rest of the string is the signer's name */
+				signer->name = g_strdup (status);
+			} else if (!strncmp (status, "BADSIG ", 7)) {
+				gpg->badsig = TRUE;
+				status += 7;
+				
+				signer = g_mime_signer_new ();
+				signer->status = GMIME_SIGNER_STATUS_BAD;
+				gpg->signer->next = signer;
+				gpg->signer = signer;
+				
+				/* get the key id of the signer */
+				status = next_token (status, &signer->keyid);
+				
+				/* the rest of the string is the signer's name */
+				signer->name = g_strdup (status);
+			} else if (!strncmp (status, "ERRSIG ", 7)) {
+				/* Note: NO_PUBKEY often comes after an ERRSIG */
+				gpg->errsig = TRUE;
+				status += 7;
+				
+				signer = g_mime_signer_new ();
+				signer->status = GMIME_SIGNER_STATUS_ERROR;
+				gpg->signer->next = signer;
+				gpg->signer = signer;
+				
+				/* get the key id of the signer */
+				status = next_token (status, &signer->keyid);
+				
+				/* skip the pubkey_algo */
+				status = next_token (status, NULL);
+				
+				/* skip the digest_algo */
+				status = next_token (status, NULL);
+				
+				/* skip the class */
+				status = next_token (status, NULL);
+				
+				/* get the signature expiration date (or 0 for never) */
+				signer->sig_expire = strtoul (status, NULL, 10);
+				status = next_token (status, NULL);
+				
+				/* the last token is the 'rc' which we don't care about */
+			} else if (!strncmp (status, "NO_PUBKEY ", 10)) {
+				/* the only token is the keyid, but we've already got it */
+				gpg->signer->errors |= GMIME_SIGNER_ERROR_NO_PUBKEY;
+				gpg->nopubkey = TRUE;
+			} else if (!strncmp (status, "EXPSIG", 6)) {
+				/* FIXME: see what else we can glean from this... */
+				gpg->signer->errors |= GMIME_SIGNER_ERROR_EXPSIG;
+			} else if (!strncmp (status, "EXPKEYSIG", 9)) {
+				gpg->signer->errors |= GMIME_SIGNER_ERROR_EXPKEYSIG;
+			} else if (!strncmp (status, "REVKEYSIG", 9)) {
+				gpg->signer->errors |= GMIME_SIGNER_ERROR_REVKEYSIG;
+			} else if (!strncmp (status, "VALIDSIG ", 9)) {
+				unsigned char *inend;
+				
 				gpg->validsig = TRUE;
-			} else if (!strncmp (status, "BADSIG", 6)) {
-				gpg->validsig = FALSE;
-			} else if (!strncmp (status, "ERRSIG", 6)) {
-				/* Note: NO_PUBKEY often comes after an ERRSIG, but do we really care? */
-				gpg->validsig = FALSE;
+				status += 9;
+				
+				signer = gpg->signer;
+				
+				/* the first token is the fingerprint */
+				status = next_token (status, &signer->fingerprint);
+				
+				/* the second token is the date the stream was signed YYYY-MM-DD */
+				status = next_token (status, NULL);
+				
+				/* the third token is the signature creation date (or 0 for unknown?) */
+				signer->sig_created = strtoul (status, (char **) &inend, 10);
+				if (inend == status || *inend != ' ')
+					break;
+				
+				status = inend + 1;
+				
+				/* the fourth token is the signature expiration date (or 0 for never) */
+				signer->sig_expire = strtoul (status, NULL, 10);
+				
+				/* ignore the rest... */
+			} else if (!strncmp (status, "TRUST_", 6)) {
+				status += 6;
+				
+				signer = gpg->signer;
+				if (!strncmp (status, "NEVER", 5)) {
+					signer->trust = GMIME_SIGNER_TRUST_NEVER;
+				} else if (!strncmp (status, "MARGINAL", 8)) {
+					signer->trust = GMIME_SIGNER_TRUST_MARGINAL;
+				} else if (!strncmp (status, "FULLY", 5)) {
+					signer->trust = GMIME_SIGNER_TRUST_FULLY;
+				} else if (!strncmp (status, "ULTIMATE", 8)) {
+					signer->trust = GMIME_SIGNER_TRUST_ULTIMATE;
+				} else if (!strncmp (status, "UNDEFINED", 9)) {
+					signer->trust = GMIME_SIGNER_TRUST_UNDEFINED;
+				}
 			}
 			break;
 		case GPG_CTX_MODE_ENCRYPT:
@@ -888,7 +1041,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 			} else if (!strncmp (status, "END_ENCRYPTION", 14)) {
 				/* nothing to do, but we know the end is near? */
 			} else if (!strncmp (status, "NO_RECP", 7)) {
-				g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_NO_VALID_RECIPIENTS,
+				g_set_error (err, GMIME_ERROR, GMIME_ERROR_NO_VALID_RECIPIENTS,
 					     _("Failed to encrypt: No valid recipients specified."));
 				return -1;
 			}
@@ -1163,12 +1316,12 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 	errno = save;
 	
 	if (diagnostics && *diagnostics) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to %s via GnuPG: %s\n\n%s"),
 			     mode, g_strerror (errno),
 			     diagnostics);
 	} else {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to %s vua GnuPG: %s\n"),
 			     mode, g_strerror (errno));
 	}
@@ -1186,11 +1339,9 @@ gpg_ctx_op_complete (struct _GpgCtx *gpg)
 static gboolean
 gpg_ctx_op_exited (struct _GpgCtx *gpg)
 {
-	pid_t retval;
 	int status;
 	
-	retval = waitpid (gpg->pid, &status, WNOHANG);
-	if (retval == gpg->pid) {
+	if (waitpid (gpg->pid, &status, WNOHANG) == gpg->pid) {
 		gpg->exit_status = status;
 		gpg->exited = TRUE;
 		return TRUE;
@@ -1203,7 +1354,6 @@ gpg_ctx_op_exited (struct _GpgCtx *gpg)
 static void
 gpg_ctx_op_cancel (struct _GpgCtx *gpg)
 {
-	pid_t retval;
 	int status;
 	
 	if (gpg->exited)
@@ -1211,8 +1361,7 @@ gpg_ctx_op_cancel (struct _GpgCtx *gpg)
 	
 	kill (gpg->pid, SIGTERM);
 	sleep (1);
-	retval = waitpid (gpg->pid, &status, WNOHANG);
-	if (retval == (pid_t) 0) {
+	if (waitpid (gpg->pid, &status, WNOHANG) == 0) {
 		/* no more mr nice guy... */
 		kill (gpg->pid, SIGKILL);
 		sleep (1);
@@ -1281,7 +1430,7 @@ gpg_sign (GMimeCipherContext *context, const char *userid, GMimeCipherHash hash,
 	gpg_ctx_set_ostream (gpg, ostream);
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to execute gpg: %s"),
 			     errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
@@ -1306,7 +1455,7 @@ gpg_sign (GMimeCipherContext *context, const char *userid, GMimeCipherHash hash,
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		errno = save;
 		
-		g_set_error (err, GMIME_ERROR_QUARK, errno, diagnostics);
+		g_set_error (err, GMIME_ERROR, errno, diagnostics);
 		gpg_ctx_free (gpg);
 		
 		return -1;
@@ -1325,21 +1474,18 @@ swrite (GMimeStream *istream)
 	char *template;
 	int fd, ret;
 	
-	template = g_strdup ("/tmp/gmime-pgp.XXXXXX");
-	fd = mkstemp (template);
-	if (fd == -1) {
+	template = g_build_filename (g_get_tmp_dir (), "gmime-pgp.XXXXXX", NULL);
+	if ((fd = mkstemp (template)) == -1) {
 		g_free (template);
 		return NULL;
 	}
 	
 	ostream = g_mime_stream_fs_new (fd);
-	ret = g_mime_stream_write_to_stream (istream, ostream);
-	if (ret != -1) {
-		ret = g_mime_stream_flush (ostream);
-		if (ret != -1)
+	if ((ret = g_mime_stream_write_to_stream (istream, ostream)) != -1) {
+		if ((ret = g_mime_stream_flush (ostream)) != -1)
 			ret = g_mime_stream_close (ostream);
 	}
-	g_mime_stream_unref (ostream);
+	g_object_unref (ostream);
 	
 	if (ret == -1) {
 		unlink (template);
@@ -1350,13 +1496,13 @@ swrite (GMimeStream *istream)
 	return template;
 }
 
-static GMimeCipherValidity *
+static GMimeSignatureValidity *
 gpg_verify (GMimeCipherContext *context, GMimeCipherHash hash,
 	    GMimeStream *istream, GMimeStream *sigstream,
 	    GError **err)
 {
 	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
-	GMimeCipherValidity *validity;
+	GMimeSignatureValidity *validity;
 	const char *diagnostics;
 	struct _GpgCtx *gpg;
 	char *sigfile = NULL;
@@ -1365,9 +1511,8 @@ gpg_verify (GMimeCipherContext *context, GMimeCipherHash hash,
 	if (sigstream != NULL) {
 		/* We are going to verify a detached signature so save
 		   the signature to a temp file. */
-		sigfile = swrite (sigstream);
-		if (sigfile == NULL) {
-			g_set_error (err, GMIME_ERROR_QUARK, errno,
+		if (!(sigfile = swrite (sigstream))) {
+			g_set_error (err, GMIME_ERROR, errno,
 				     _("Cannot verify message signature: "
 				       "could not create temp file: %s"),
 				     g_strerror (errno));
@@ -1382,7 +1527,7 @@ gpg_verify (GMimeCipherContext *context, GMimeCipherHash hash,
 	gpg_ctx_set_istream (gpg, istream);
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to execute gpg: %s"),
 			     errno ? g_strerror (errno) : _("Unknown"));
 		goto exception;
@@ -1396,10 +1541,26 @@ gpg_verify (GMimeCipherContext *context, GMimeCipherHash hash,
 	}
 	
 	valid = gpg_ctx_op_wait (gpg) == 0;
-	validity = g_mime_cipher_validity_new ();
 	diagnostics = gpg_ctx_get_diagnostics (gpg);
-	g_mime_cipher_validity_set_valid (validity, valid);
-	g_mime_cipher_validity_set_description (validity, diagnostics);
+	
+	validity = g_mime_signature_validity_new ();
+	g_mime_signature_validity_set_details (validity, diagnostics);
+	
+	if (gpg->goodsig && !(gpg->badsig || gpg->errsig || gpg->nodata)) {
+		/* all signatures were good */
+		validity->status = GMIME_SIGNATURE_STATUS_GOOD;
+	} else if (gpg->badsig && !(gpg->goodsig && !gpg->errsig)) {
+		/* all signatures were bad */
+		validity->status = GMIME_SIGNATURE_STATUS_BAD;
+	} else if (!gpg->nodata) {
+		validity->status = GMIME_SIGNATURE_STATUS_UNKNOWN;
+	} else {
+		validity->status = GMIME_SIGNATURE_STATUS_BAD;
+	}
+	
+	validity->signers = gpg->signers;
+	gpg->signers = NULL;
+	
 	gpg_ctx_free (gpg);
 	
 	if (sigfile) {
@@ -1439,12 +1600,11 @@ gpg_encrypt (GMimeCipherContext *context, gboolean sign, const char *userid,
 	gpg_ctx_set_ostream (gpg, ostream);
 	gpg_ctx_set_always_trust (gpg, ctx->always_trust);
 	
-	for (i = 0; i < recipients->len; i++) {
+	for (i = 0; i < recipients->len; i++)
 		gpg_ctx_add_recipient (gpg, recipients->pdata[i]);
-	}
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to execute gpg: %s"),
 			     errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
@@ -1469,7 +1629,7 @@ gpg_encrypt (GMimeCipherContext *context, gboolean sign, const char *userid,
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		errno = save;
 		
-		g_set_error (err, GMIME_ERROR_QUARK, errno, diagnostics);
+		g_set_error (err, GMIME_ERROR, errno, diagnostics);
 		gpg_ctx_free (gpg);
 		
 		return -1;
@@ -1494,7 +1654,7 @@ gpg_decrypt (GMimeCipherContext *context, GMimeStream *istream,
 	gpg_ctx_set_ostream (gpg, ostream);
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to execute gpg: %s"),
 			     errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
@@ -1519,7 +1679,7 @@ gpg_decrypt (GMimeCipherContext *context, GMimeStream *istream,
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		errno = save;
 		
-		g_set_error (err, GMIME_ERROR_QUARK, errno, diagnostics);
+		g_set_error (err, GMIME_ERROR, errno, diagnostics);
 		gpg_ctx_free (gpg);
 		
 		return -1;
@@ -1541,7 +1701,7 @@ gpg_import_keys (GMimeCipherContext *context, GMimeStream *istream, GError **err
 	gpg_ctx_set_istream (gpg, istream);
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to execute gpg: %s"),
 			     errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
@@ -1566,7 +1726,7 @@ gpg_import_keys (GMimeCipherContext *context, GMimeStream *istream, GError **err
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		errno = save;
 		
-		g_set_error (err, GMIME_ERROR_QUARK, errno, diagnostics);
+		g_set_error (err, GMIME_ERROR, errno, diagnostics);
 		gpg_ctx_free (gpg);
 		
 		return -1;
@@ -1594,7 +1754,7 @@ gpg_export_keys (GMimeCipherContext *context, GPtrArray *keys, GMimeStream *ostr
 	}
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to execute gpg: %s"),
 			     errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
@@ -1619,7 +1779,7 @@ gpg_export_keys (GMimeCipherContext *context, GPtrArray *keys, GMimeStream *ostr
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		errno = save;
 		
-		g_set_error (err, GMIME_ERROR_QUARK, errno, diagnostics);
+		g_set_error (err, GMIME_ERROR, errno, diagnostics);
 		gpg_ctx_free (gpg);
 		
 		return -1;
