@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -28,10 +25,12 @@
 #include <string.h>
 #include <ctype.h>
 
+#include "gmime-common.h"
 #include "gmime-header.h"
 #include "gmime-utils.h"
 #include "gmime-stream-mem.h"
 
+
 struct raw_header {
 	struct raw_header *next;
 	char *name;
@@ -42,29 +41,10 @@ struct _GMimeHeader {
 	GHashTable *hash;
 	GHashTable *writers;
 	struct raw_header *headers;
+	char *raw;
 };
 
 
-static int
-header_equal (gconstpointer v, gconstpointer v2)
-{
-	return g_strcasecmp ((const char *) v, (const char *) v2) == 0;
-}
-
-static guint
-header_hash (gconstpointer key)
-{
-	const char *p = key;
-	guint h = tolower (*p);
-	
-	if (h)
-		for (p += 1; *p != '\0'; p++)
-			h = (h << 5) - h + tolower (*p);
-	
-	return h;
-}
-
-
 /**
  * g_mime_header_new:
  *
@@ -78,9 +58,10 @@ g_mime_header_new ()
 	GMimeHeader *new;
 	
 	new = g_new (GMimeHeader, 1);
-	new->hash = g_hash_table_new (header_hash, header_equal);
-	new->writers = g_hash_table_new (header_hash, header_equal);
+	new->hash = g_hash_table_new (g_mime_strcase_hash, g_mime_strcase_equal);
+	new->writers = g_hash_table_new (g_mime_strcase_hash, g_mime_strcase_equal);
 	new->headers = NULL;
+	new->raw = NULL;
 	
 	return new;
 }
@@ -117,6 +98,7 @@ g_mime_header_destroy (GMimeHeader *header)
 		g_hash_table_destroy (header->hash);
 		g_hash_table_foreach (header->writers, writer_free, NULL);
 		g_hash_table_destroy (header->writers);
+		g_free (header->raw);
 		g_free (header);
 	}
 }
@@ -161,6 +143,9 @@ g_mime_header_set (GMimeHeader *header, const char *name, const char *value)
 		
 		g_hash_table_insert (header->hash, n->name, n);
 	}
+	
+	g_free (header->raw);
+	header->raw = NULL;
 }
 
 
@@ -187,7 +172,9 @@ g_mime_header_add (GMimeHeader *header, const char *name, const char *value)
 	n->name = g_strdup (name);
 	n->value = g_strdup (value);
 	
-	for (h = header->headers; h && h->next; h = h->next);
+	h = header->headers;
+	while (h && h->next)
+		h = h->next;
 	
 	if (h)
 		h->next = n;
@@ -196,6 +183,42 @@ g_mime_header_add (GMimeHeader *header, const char *name, const char *value)
 	
 	if (!g_hash_table_lookup (header->hash, name))
 		g_hash_table_insert (header->hash, n->name, n);
+	
+	g_free (header->raw);
+	header->raw = NULL;
+}
+
+
+/**
+ * g_mime_header_prepend:
+ * @header: header object
+ * @name: header name
+ * @value: header value
+ *
+ * Adds a header to the head of the list. If @value is %NULL, a space
+ * will be set aside for it (useful for setting the order of headers
+ * before values can be obtained for them) otherwise the header will
+ * be unset.
+ **/
+void
+g_mime_header_prepend (GMimeHeader *header, const char *name, const char *value)
+{
+	struct raw_header *n;
+	
+	g_return_if_fail (header != NULL);
+	g_return_if_fail (name != NULL);
+	
+	n = g_new (struct raw_header, 1);
+	n->next = header->headers;
+	n->name = g_strdup (name);
+	n->value = g_strdup (value);
+	header->headers = n;
+	
+	if (!g_hash_table_lookup (header->hash, name))
+		g_hash_table_insert (header->hash, n->name, n);
+	
+	g_free (header->raw);
+	header->raw = NULL;
 }
 
 
@@ -255,6 +278,9 @@ g_mime_header_remove (GMimeHeader *header, const char *name)
 		g_free (h->value);
 		g_free (h);
 	}
+	
+	g_free (header->raw);
+	header->raw = NULL;
 }
 
 
@@ -291,6 +317,9 @@ g_mime_header_write_to_stream (const GMimeHeader *header, GMimeStream *stream)
 	g_return_val_if_fail (header != NULL, -1);
 	g_return_val_if_fail (stream != NULL, -1);
 	
+	if (header->raw)
+		return g_mime_stream_write_string (stream, header->raw);
+	
 	h = header->headers;
 	while (h) {
 		if (h->value) {
@@ -335,7 +364,8 @@ g_mime_header_to_string (const GMimeHeader *header)
 	stream = g_mime_stream_mem_new ();
 	g_mime_stream_mem_set_byte_array (GMIME_STREAM_MEM (stream), array);
 	g_mime_header_write_to_stream (header, stream);
-	g_mime_stream_unref (stream);
+	g_object_unref (stream);
+	
 	g_byte_array_append (array, "", 1);
 	str = array->data;
 	g_byte_array_free (array, FALSE);
@@ -393,3 +423,38 @@ g_mime_header_register_writer (GMimeHeader *header, const char *name, GMimeHeade
 	if (writer)
 		g_hash_table_insert (header->writers, g_strdup (name), writer);
 }
+
+
+
+/**
+ * g_mime_header_set_raw:
+ * @header: header object
+ * @raw: raw mime part header
+ *
+ * Set the raw header.
+ **/
+void
+g_mime_header_set_raw (GMimeHeader *header, const char *raw)
+{
+	g_return_if_fail (header != NULL);
+	
+	g_free (header->raw);
+	header->raw = raw ? g_strdup (raw) : NULL;
+}
+
+
+/**
+ * g_mime_header_has_raw:
+ * @header: ehader object
+ *
+ * Gets whether or not a raw header has been set on @header.
+ *
+ * Returns %TRUE if a raw header is set or %FALSE otherwise.
+ **/
+gboolean
+g_mime_header_has_raw (GMimeHeader *header)
+{
+	g_return_val_if_fail (header != NULL, FALSE);
+	
+	return header->raw ? TRUE : FALSE;
+}
