@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2000-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -55,6 +52,7 @@ static void message_set_reply_to (GMimeMessage *message, const char *reply_to);
 static void message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string);
 static void message_set_subject (GMimeMessage *message, const char *subject);
 
+static ssize_t write_addrspec (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_received (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_subject (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_msgid (GMimeStream *stream, const char *name, const char *value);
@@ -133,9 +131,22 @@ g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass)
 	message->message_id = NULL;
 	message->mime_part = NULL;
 	
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Sender", write_addrspec);
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "From", write_addrspec);
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "To", write_addrspec);
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Cc", write_addrspec);
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Bcc", write_addrspec);
+	
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-Sender", write_addrspec);
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-From", write_addrspec);
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-To", write_addrspec);
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-Cc", write_addrspec);
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-Bcc", write_addrspec);
+	
 	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Subject", write_subject);
 	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Received", write_received);
 	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Message-Id", write_msgid);
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "References", write_addrspec);
 }
 
 static gboolean
@@ -537,6 +548,22 @@ write_msgid (GMimeStream *stream, const char *name, const char *value)
 	return g_mime_stream_printf (stream, "%s: %s\n", name, value);
 }
 
+static ssize_t
+write_addrspec (GMimeStream *stream, const char *name, const char *value)
+{
+	char *unfolded, *folded;
+	ssize_t n;
+	
+	unfolded = g_strdup_printf ("%s: %s\n", name, value);
+	folded = g_mime_utils_structured_header_fold (unfolded);
+	g_free (unfolded);
+	
+	n = g_mime_stream_write_string (stream, folded);
+	g_free (folded);
+	
+	return n;
+}
+
 
 enum {
 	HEADER_FROM,
@@ -566,20 +593,27 @@ static gboolean
 process_header (GMimeObject *object, const char *header, const char *value)
 {
 	GMimeMessage *message = (GMimeMessage *) object;	
+	InternetAddressList *addrlist;
 	int offset, i;
 	time_t date;
 	
 	for (i = 0; headers[i]; i++) {
-		if (!strcasecmp (headers[i], header))
+		if (!g_ascii_strcasecmp (headers[i], header))
 			break;
 	}
 	
 	switch (i) {
 	case HEADER_FROM:
-		message_set_sender (message, value);
+		g_free (message->from);
+		addrlist = internet_address_parse_string (value);
+		message->from = internet_address_list_to_string (addrlist, FALSE);
+		internet_address_list_destroy (addrlist);
 		break;
 	case HEADER_REPLY_TO:
-		message_set_reply_to (message, value);
+		g_free (message->reply_to);
+		addrlist = internet_address_parse_string (value);
+		message->reply_to = internet_address_list_to_string (addrlist, FALSE);
+		internet_address_list_destroy (addrlist);
 		break;
 	case HEADER_TO:
 		message_add_recipients_from_string (message, GMIME_RECIPIENT_TYPE_TO, value);
@@ -591,7 +625,8 @@ process_header (GMimeObject *object, const char *header, const char *value)
 		message_add_recipients_from_string (message, GMIME_RECIPIENT_TYPE_BCC, value);
 		break;
 	case HEADER_SUBJECT:
-		message_set_subject (message, value);
+		g_free (message->subject);
+		message->subject = g_mime_utils_header_decode_text (value);
 		break;
 	case HEADER_DATE:
 		if (value) {
@@ -615,35 +650,41 @@ process_header (GMimeObject *object, const char *header, const char *value)
 static void
 message_add_header (GMimeObject *object, const char *header, const char *value)
 {
-	if (!strcasecmp ("MIME-Version", header))
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	
-	if (strncasecmp ("Content-", header, 8)) {
+	if (g_ascii_strncasecmp ("Content-", header, 8)) {
 		if (process_header (object, header, value))
 			g_mime_header_add (object->headers, header, value);
 		else
 			GMIME_OBJECT_CLASS (parent_class)->add_header (object, header, value);
 	}
+	
+	if (((GMimeMessage *) object)->mime_part)
+		g_mime_header_set_raw (((GMimeMessage *) object)->mime_part->headers, NULL);
 }
 
 static void
 message_set_header (GMimeObject *object, const char *header, const char *value)
 {
-	if (!strcasecmp ("MIME-Version", header))
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	
-	if (strncasecmp ("Content-", header, 8)) {
+	if (g_ascii_strncasecmp ("Content-", header, 8)) {
 		if (process_header (object, header, value))
 			g_mime_header_set (object->headers, header, value);
 		else
 			GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
 	}
+	
+	if (((GMimeMessage *) object)->mime_part)
+		g_mime_header_set_raw (((GMimeMessage *) object)->mime_part->headers, NULL);
 }
 
 static const char *
@@ -652,11 +693,13 @@ message_get_header (GMimeObject *object, const char *header)
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	
-	if (!strcasecmp ("MIME-Version", header))
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return "1.0";
 	
-	if (strncasecmp ("Content-", header, 8))
+	if (g_ascii_strncasecmp ("Content-", header, 8) != 0)
 		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
+	else if (((GMimeMessage *) object)->mime_part)
+		return g_mime_object_get_header (((GMimeMessage *) object)->mime_part, header);
 	else
 		return NULL;
 }
@@ -668,17 +711,17 @@ message_remove_header (GMimeObject *object, const char *header)
 	InternetAddressList *addrlist;
 	int i;
 	
-	if (!strcasecmp ("MIME-Version", header))
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		return;
 	
 	for (i = 0; headers[i]; i++) {
-		if (!strcasecmp (headers[i], header))
+		if (!g_ascii_strcasecmp (headers[i], header))
 			break;
 	}
 	
@@ -723,6 +766,9 @@ message_remove_header (GMimeObject *object, const char *header)
 	}
 	
 	GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
+	
+	if (((GMimeMessage *) object)->mime_part)
+		g_mime_header_set_raw (((GMimeMessage *) object)->mime_part->headers, NULL);
 }
 
 
@@ -738,10 +784,15 @@ message_get_headers (GMimeObject *object)
 	stream = g_mime_stream_mem_new ();
 	g_mime_stream_mem_set_byte_array (GMIME_STREAM_MEM (stream), ba);
 	
-	g_mime_header_write_to_stream (object->headers, stream);
-	if (message->mime_part) {
-		g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
+	if (message->mime_part && g_mime_header_has_raw (message->mime_part->headers)) {
+		/* if the mime part has raw headers, then it contains the message headers as well */
 		g_mime_header_write_to_stream (message->mime_part->headers, stream);
+	} else {
+		g_mime_header_write_to_stream (object->headers, stream);
+		if (message->mime_part) {
+			g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
+			g_mime_header_write_to_stream (message->mime_part->headers, stream);
+		}
 	}
 	
 	g_object_unref (stream);
@@ -758,18 +809,22 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	GMimeMessage *message = (GMimeMessage *) object;
 	ssize_t nwritten, total = 0;
 	
-	/* write the content headers */
-	if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
-		return -1;
-	
-	total += nwritten;
-	
-	if (message->mime_part) {
-		if ((nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n")) == -1)
+	if (!(message->mime_part && g_mime_header_has_raw (message->mime_part->headers))) {
+		/* write the content headers */
+		if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
 			return -1;
 		
 		total += nwritten;
 		
+		if (message->mime_part) {
+			if ((nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n")) == -1)
+				return -1;
+			
+			total += nwritten;
+		}
+	}
+	
+	if (message->mime_part) {
 		nwritten = g_mime_object_write_to_stream (message->mime_part, stream);
 	} else {
 		if ((nwritten = g_mime_stream_write (stream, "\n", 1)) == -1)
@@ -811,16 +866,6 @@ g_mime_message_new (gboolean pretty_headers)
 }
 
 
-static void
-message_set_sender (GMimeMessage *message, const char *sender)
-{
-	if (message->from)
-		g_free (message->from);
-	
-	message->from = g_strstrip (g_strdup (sender));
-}
-
-
 /**
  * g_mime_message_set_sender:
  * @message: MIME Message to change
@@ -832,11 +877,21 @@ message_set_sender (GMimeMessage *message, const char *sender)
 void
 g_mime_message_set_sender (GMimeMessage *message, const char *sender)
 {
+	InternetAddressList *addrlist;
+	char *encoded;
+	
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (sender != NULL);
 	
-	message_set_sender (message, sender);
-	g_mime_header_set (GMIME_OBJECT (message)->headers, "From", message->from);
+	g_free (message->from);
+	
+	addrlist = internet_address_parse_string (sender);
+	message->from = internet_address_list_to_string (addrlist, FALSE);
+	encoded = internet_address_list_to_string (addrlist, TRUE);
+	internet_address_list_destroy (addrlist);
+	
+	g_mime_header_set (GMIME_OBJECT (message)->headers, "From", encoded);
+	g_free (encoded);
 }
 
 
@@ -857,16 +912,6 @@ g_mime_message_get_sender (GMimeMessage *message)
 }
 
 
-static void
-message_set_reply_to (GMimeMessage *message, const char *reply_to)
-{
-	if (message->reply_to)
-		g_free (message->reply_to);
-	
-	message->reply_to = g_strstrip (g_strdup (reply_to));
-}
-
-
 /**
  * g_mime_message_set_reply_to:
  * @message: MIME Message to change
@@ -880,7 +925,9 @@ g_mime_message_set_reply_to (GMimeMessage *message, const char *reply_to)
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (reply_to != NULL);
 	
-	message_set_reply_to (message, reply_to);
+	g_free (message->reply_to);
+	message->reply_to = g_strstrip (g_strdup (reply_to));
+	
 	g_mime_header_set (GMIME_OBJECT (message)->headers, "Reply-To", message->reply_to);
 }
 
@@ -1020,16 +1067,6 @@ g_mime_message_get_recipients (GMimeMessage *message, const char *type)
 }
 
 
-static void
-message_set_subject (GMimeMessage *message, const char *subject)
-{
-	if (message->subject)
-		g_free (message->subject);
-	
-	message->subject = g_strstrip (g_strdup (subject));
-}
-
-
 /**
  * g_mime_message_set_subject:
  * @message: MIME Message
@@ -1045,7 +1082,8 @@ g_mime_message_set_subject (GMimeMessage *message, const char *subject)
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (subject != NULL);
 	
-	message_set_subject (message, subject);
+	g_free (message->subject);
+	message->subject = g_strstrip (g_strdup (subject));
 	
 	encoded = g_mime_utils_header_encode_text (message->subject);
 	g_mime_header_set (GMIME_OBJECT (message)->headers, "Subject", encoded);
@@ -1153,9 +1191,7 @@ g_mime_message_set_message_id (GMimeMessage *message, const char *message_id)
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (message_id != NULL);
 	
-	if (message->message_id)
-		g_free (message->message_id);
-	
+	g_free (message->message_id);
 	message->message_id = g_strstrip (g_strdup (message_id));
 	
 	msgid = g_strdup_printf ("<%s>", message_id);
@@ -1276,6 +1312,7 @@ g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
 	g_object_ref (mime_part);
+	g_mime_header_set_raw (mime_part->headers, NULL);
 	
 	if (message->mime_part)
 		g_object_unref (message->mime_part);
@@ -1351,8 +1388,8 @@ handle_multipart_alternative (GMimeMultipart *multipart, gboolean want_plain, gb
 		
 		type = g_mime_object_get_content_type (mime_part);
 		if (g_mime_content_type_is_type (type, "text", "*")) {
-			if (!text_part || !strcasecmp (type->subtype, want_plain ? "plain" : "html")) {
-				*is_html = !strcasecmp (type->subtype, "html");
+			if (!text_part || !g_ascii_strcasecmp (type->subtype, want_plain ? "plain" : "html")) {
+				*is_html = !g_ascii_strcasecmp (type->subtype, "html");
 				text_part = mime_part;
 			}
 		}
@@ -1387,18 +1424,17 @@ handle_multipart_mixed (GMimeMultipart *multipart, gboolean want_plain, gboolean
 					text_part = mime_part;
 			}
 		} else if (g_mime_content_type_is_type (type, "text", "*")) {
-			if (!strcasecmp (type->subtype, want_plain ? "plain" : "html")) {
+			if (!g_ascii_strcasecmp (type->subtype, want_plain ? "plain" : "html")) {
 				/* we got what we came for */
-				*is_html = !strcasecmp (type->subtype, "html");
+				*is_html = !g_ascii_strcasecmp (type->subtype, "html");
 				return mime_part;
 			}
 			
-			/* if we haven't yet found a text part or if
-                           it is a type we can understand and it is
-                           the first of that type, save it */
-			if (!text_part || (!strcasecmp (type->subtype, "plain") && (first_type &&
-					   strcasecmp (type->subtype, first_type->subtype) != 0))) {
-				*is_html = !strcasecmp (type->subtype, "html");
+			/* if we haven't yet found a text part or if it is a type we can
+			 * understand and it is the first of that type, save it */
+			if (!text_part || (!g_ascii_strcasecmp (type->subtype, "plain") && (first_type &&
+					   g_ascii_strcasecmp (type->subtype, first_type->subtype) != 0))) {
+				*is_html = !g_ascii_strcasecmp (type->subtype, "html");
 				text_part = mime_part;
 				first_type = type;
 			}
@@ -1443,7 +1479,7 @@ g_mime_message_get_body (const GMimeMessage *message, gboolean want_plain, gbool
 	
 	type = g_mime_object_get_content_type (message->mime_part);
 	if (GMIME_IS_MULTIPART (message->mime_part)) {
-		/* lets see if we can find a body in the multipart */
+		/* let's see if we can find a body in the multipart */
 		multipart = GMIME_MULTIPART (message->mime_part);
 		if (g_mime_content_type_is_type (type, "multipart", "alternative"))
 			mime_part = handle_multipart_alternative (multipart, want_plain, is_html);
