@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -28,17 +25,14 @@
 
 #include <gmime/gmime-object.h>
 
-#ifdef __cplusplus
-extern "C" {
-#pragma }
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 #define GMIME_TYPE_MULTIPART            (g_mime_multipart_get_type ())
-#define GMIME_MULTIPART(obj)            (GMIME_CHECK_CAST ((obj), GMIME_TYPE_MULTIPART, GMimeMultipart))
-#define GMIME_MULTIPART_CLASS(klass)    (GMIME_CHECK_CLASS_CAST ((klass), GMIME_TYPE_MULTIPART, GMimeMultipartClass))
-#define GMIME_IS_MULTIPART(obj)         (GMIME_CHECK_TYPE ((obj), GMIME_TYPE_MULTIPART))
-#define GMIME_IS_MULTIPART_CLASS(klass) (GMIME_CHECK_CLASS_TYPE ((klass), GMIME_TYPE_MULTIPART))
-#define GMIME_MULTIPART_GET_CLASS(obj)  (GMIME_CHECK_GET_CLASS ((obj), GMIME_TYPE_MULTIPART, GMimeMultipartClass))
+#define GMIME_MULTIPART(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GMIME_TYPE_MULTIPART, GMimeMultipart))
+#define GMIME_MULTIPART_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GMIME_TYPE_MULTIPART, GMimeMultipartClass))
+#define GMIME_IS_MULTIPART(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GMIME_TYPE_MULTIPART))
+#define GMIME_IS_MULTIPART_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GMIME_TYPE_MULTIPART))
+#define GMIME_MULTIPART_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GMIME_TYPE_MULTIPART, GMimeMultipartClass))
 
 typedef struct _GMimeMultipart GMimeMultipart;
 typedef struct _GMimeMultipartClass GMimeMultipartClass;
@@ -57,17 +51,17 @@ struct _GMimeMultipart {
 struct _GMimeMultipartClass {
 	GMimeObjectClass parent_class;
 	
-	void (*add_part) (GMimeMultipart *multipart, GMimeObject *part);
-	void (*add_part_at) (GMimeMultipart *multipart, GMimeObject *part, int index);
-	void (*remove_part) (GMimeMultipart *multipart, GMimeObject *part);
+	void (* add_part) (GMimeMultipart *multipart, GMimeObject *part);
+	void (* add_part_at) (GMimeMultipart *multipart, GMimeObject *part, int index);
+	void (* remove_part) (GMimeMultipart *multipart, GMimeObject *part);
 	
-	GMimeObject * (*remove_part_at) (GMimeMultipart *multipart, int index);
-	GMimeObject * (*get_part) (GMimeMultipart *multipart, int index);
+	GMimeObject * (* remove_part_at) (GMimeMultipart *multipart, int index);
+	GMimeObject * (* get_part) (GMimeMultipart *multipart, int index);
 	
-	int  (*get_number) (GMimeMultipart *multipart);
+	int  (* get_number) (GMimeMultipart *multipart);
 	
-	void (*set_boundary) (GMimeMultipart *multipart, const char *boundary);
-	const char * (*get_boundary) (GMimeMultipart *multipart);
+	void (* set_boundary) (GMimeMultipart *multipart, const char *boundary);
+	const char * (* get_boundary) (GMimeMultipart *multipart);
 };
 
 
@@ -100,8 +94,6 @@ void g_mime_multipart_foreach (GMimeMultipart *multipart, GMimePartFunc callback
 GMimeObject *g_mime_multipart_get_subpart_from_content_id (GMimeMultipart *multipart,
 							   const char *content_id);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __GMIME_MULTIPART_H__ */
