@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -28,6 +25,7 @@
 #include <ctype.h>
 #include <string.h>
 
+#include "gmime-common.h"
 #include "gmime-object.h"
 #include "gmime-stream-mem.h"
 #include "gmime-utils.h"
@@ -56,10 +54,11 @@ static void set_content_type (GMimeObject *object, GMimeContentType *content_typ
 static char *get_headers (GMimeObject *object);
 static ssize_t write_to_stream (GMimeObject *object, GMimeStream *stream);
 
-static int strcase_equal (gconstpointer v, gconstpointer v2);
-static guint strcase_hash (gconstpointer key);
+static ssize_t write_content_type (GMimeStream *stream, const char *name, const char *value);
+
 static void type_registry_init (void);
 
+
 static GHashTable *type_hash = NULL;
 
 static GObjectClass *parent_class = NULL;
@@ -117,6 +116,8 @@ g_mime_object_init (GMimeObject *object, GMimeObjectClass *klass)
 	object->content_type = NULL;
 	object->headers = g_mime_header_new ();
 	object->content_id = NULL;
+	
+	g_mime_header_register_writer (object->headers, "Content-Type", write_content_type);
 }
 
 static void
@@ -136,18 +137,47 @@ g_mime_object_finalize (GObject *object)
 }
 
 
+static ssize_t
+write_content_type (GMimeStream *stream, const char *name, const char *value)
+{
+	GMimeContentType *content_type;
+	ssize_t nwritten;
+	GString *out;
+	char *val;
+	
+	out = g_string_new ("");
+	g_string_printf (out, "%s: ", name);
+	
+	content_type = g_mime_content_type_new_from_string (value);
+	
+	val = g_mime_content_type_to_string (content_type);
+	g_string_append (out, val);
+	g_free (val);
+	
+	g_mime_param_write_to_string (content_type->params, TRUE, out);
+	g_mime_content_type_destroy (content_type);
+	
+	nwritten = g_mime_stream_write (stream, out->str, out->len);
+	g_string_free (out, TRUE);
+	
+	return nwritten;
+}
+
+
 /**
  * g_mime_object_ref:
  * @object: mime object
  *
  * Ref's a MIME object.
+ *
+ * WARNING: This method is deprecated. Use g_object_ref() instead.
  **/
 void
 g_mime_object_ref (GMimeObject *object)
 {
 	g_return_if_fail (GMIME_IS_OBJECT (object));
 	
-	g_object_ref ((GObject *) object);
+	g_object_ref (object);
 }
 
 
@@ -156,13 +186,15 @@ g_mime_object_ref (GMimeObject *object)
  * @object: mime object
  *
  * Unref's a MIME object.
+ *
+ * WARNING: This method is deprecated. Use g_object_unref() instead.
  **/
 void
 g_mime_object_unref (GMimeObject *object)
 {
 	g_return_if_fail (GMIME_IS_OBJECT (object));
 	
-	g_object_unref ((GObject *) object);
+	g_object_unref (object);
 }
 
 
@@ -173,7 +205,7 @@ g_mime_object_unref (GMimeObject *object)
  * @object_type: object type
  *
  * Registers the object type @object_type for use with the
- * #g_mime_object_new_type convenience function.
+ * g_mime_object_new_type() convenience function.
  *
  * Note: You may use the wildcard "*" to match any type and/or
  * subtype.
@@ -190,12 +222,11 @@ g_mime_object_register_type (const char *type, const char *subtype, GType object
 	
 	type_registry_init ();
 	
-	bucket = g_hash_table_lookup (type_hash, type);
-	if (!bucket) {
+	if (!(bucket = g_hash_table_lookup (type_hash, type))) {
 		bucket = g_new (struct _type_bucket, 1);
 		bucket->type = g_strdup (type);
 		bucket->object_type = *type == '*' ? object_type : 0;
-		bucket->subtype_hash = g_hash_table_new (strcase_hash, strcase_equal);
+		bucket->subtype_hash = g_hash_table_new (g_mime_strcase_hash, g_mime_strcase_equal);
 		g_hash_table_insert (type_hash, bucket->type, bucket);
 	}
 	
@@ -219,7 +250,7 @@ init (GMimeObject *object)
  * @subtype: mime subtype
  *
  * Performs a lookup of registered #GMimeObject subclasses, registered
- * using #g_mime_object_register_type, to find an appropriate class
+ * using g_mime_object_register_type(), to find an appropriate class
  * capable of handling MIME parts of type @type/@subtype. If no class
  * has been registered to handle that type, it looks for a registered
  * class that can handle @type. If that also fails, then it will use
@@ -240,21 +271,19 @@ g_mime_object_new_type (const char *type, const char *subtype)
 	
 	type_registry_init ();
 	
-	bucket = g_hash_table_lookup (type_hash, type);
-	if (!bucket) {
-		bucket = g_hash_table_lookup (type_hash, "*");
-		obj_type = bucket ? bucket->object_type : 0;
-	} else {
+	if ((bucket = g_hash_table_lookup (type_hash, type))) {
 		if (!(sub = g_hash_table_lookup (bucket->subtype_hash, subtype)))
 			sub = g_hash_table_lookup (bucket->subtype_hash, "*");
 		
 		obj_type = sub ? sub->object_type : 0;
+	} else {
+		bucket = g_hash_table_lookup (type_hash, "*");
+		obj_type = bucket ? bucket->object_type : 0;
 	}
 	
 	if (!obj_type) {
 		/* use the default mime object */
-		bucket = g_hash_table_lookup (type_hash, "*");
-		if (bucket) {
+		if ((bucket = g_hash_table_lookup (type_hash, "*"))) {
 			sub = g_hash_table_lookup (bucket->subtype_hash, "*");
 			obj_type = sub ? sub->object_type : 0;
 		}
@@ -287,8 +316,7 @@ sync_content_type (GMimeObject *object)
 	g_string_append (string, type);
 	g_free (type);
 	
-	params = content_type->params;
-	if (params)
+	if ((params = content_type->params))
 		g_mime_param_write_to_string (params, FALSE, string);
 	
 	p = string->str;
@@ -440,38 +468,40 @@ static char *headers[] = {
 	NULL
 };
 
-static void
+static gboolean
 process_header (GMimeObject *object, const char *header, const char *value)
 {
 	GMimeContentType *content_type;
 	int i;
 	
 	for (i = 0; headers[i]; i++) {
-		if (!g_strcasecmp (headers[i], header))
+		if (!g_ascii_strcasecmp (headers[i], header))
 			break;
 	}
 	
 	switch (i) {
 	case HEADER_CONTENT_TYPE:
 		content_type = g_mime_content_type_new_from_string (value);
-		if (object->content_type)
-			g_mime_content_type_destroy (object->content_type);
-		object->content_type = content_type;
+		g_mime_object_set_content_type (object, content_type);
 		break;
 	case HEADER_CONTENT_ID:
 		g_free (object->content_id);
 		object->content_id = g_mime_utils_decode_message_id (value);
 		break;
 	default:
-		break;
+		return FALSE;
 	}
+	
+	g_mime_header_set (object->headers, header, value);
+	
+	return TRUE;
 }
 
 static void
 add_header (GMimeObject *object, const char *header, const char *value)
 {
-	process_header (object, header, value);
-	g_mime_header_add (object->headers, header, value);
+	if (!process_header (object, header, value))
+		g_mime_header_add (object->headers, header, value);
 }
 
 
@@ -497,8 +527,8 @@ g_mime_object_add_header (GMimeObject *object, const char *header, const char *v
 static void
 set_header (GMimeObject *object, const char *header, const char *value)
 {
-	process_header (object, header, value);
-	g_mime_header_set (object->headers, header, value);
+	if (!process_header (object, header, value))
+		g_mime_header_set (object->headers, header, value);
 }
 
 
@@ -648,7 +678,7 @@ g_mime_object_to_string (GMimeObject *object)
 	
 	g_mime_object_write_to_stream (object, stream);
 	
-	g_mime_stream_unref (stream);
+	g_object_unref (stream);
 	g_byte_array_append (array, "", 1);
 	str = array->data;
 	g_byte_array_free (array, FALSE);
@@ -657,29 +687,13 @@ g_mime_object_to_string (GMimeObject *object)
 }
 
 
-static int
-strcase_equal (gconstpointer v, gconstpointer v2)
-{
-	return g_strcasecmp ((const char *) v, (const char *) v2) == 0;
-}
-
-static guint
-strcase_hash (gconstpointer key)
-{
-	const char *p = key;
-	guint h = tolower (*p);
-	
-	if (h)
-		for (p += 1; *p != '\0'; p++)
-			h = (h << 5) - h + tolower (*p);
-	
-	return h;
-}
-
 static void
 subtype_bucket_foreach (gpointer key, gpointer value, gpointer user_data)
 {
-	g_free (key);
+	struct _subtype_bucket *bucket = value;
+	
+	g_free (bucket->subtype);
+	g_free (bucket);
 }
 
 static void
@@ -710,7 +724,7 @@ type_registry_init (void)
 	if (type_hash)
 		return;
 	
-	type_hash = g_hash_table_new (strcase_hash, strcase_equal);
+	type_hash = g_hash_table_new (g_mime_strcase_hash, g_mime_strcase_equal);
 	
 	g_atexit (type_registry_shutdown);
 }
