@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -32,7 +29,7 @@ static void g_mime_stream_cat_init (GMimeStreamCat *stream, GMimeStreamCatClass
 static void g_mime_stream_cat_finalize (GObject *object);
 
 static ssize_t stream_read (GMimeStream *stream, char *buf, size_t len);
-static ssize_t stream_write (GMimeStream *stream, char *buf, size_t len);
+static ssize_t stream_write (GMimeStream *stream, const char *buf, size_t len);
 static int stream_flush (GMimeStream *stream);
 static int stream_close (GMimeStream *stream);
 static gboolean stream_eos (GMimeStream *stream);
@@ -115,7 +112,7 @@ g_mime_stream_cat_finalize (GObject *object)
 	p = cat->sources;
 	while (p) {
 		n = p->next;
-		g_mime_stream_unref (p->stream);
+		g_object_unref (p->stream);
 		g_free (p);
 		p = n;
 	}
@@ -175,7 +172,7 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 }
  
 static ssize_t
-stream_write (GMimeStream *stream, char *buf, size_t len)
+stream_write (GMimeStream *stream, const char *buf, size_t len)
 {
 	GMimeStreamCat *cat = (GMimeStreamCat *) stream;
 	struct _cat_node *current;
@@ -199,7 +196,7 @@ stream_write (GMimeStream *stream, char *buf, size_t len)
 	do {
 		n = -1;
 		while (!g_mime_stream_eos (current->stream) && nwritten < len) {
-			n = g_mime_stream_read (current->stream, buf + nwritten, len - nwritten);
+			n = g_mime_stream_write (current->stream, buf + nwritten, len - nwritten);
 			if (n > 0)
 				nwritten += n;
 		}
@@ -326,7 +323,23 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 			real += current->length;
 		
 		real += current->stream->bound_start;
-		ret = g_mime_stream_seek (current->stream, real, GMIME_STREAM_SEEK_SET);
+		
+		if (current->stream->position != real) {
+			if (real > current->stream->bound_start) {
+				/* no choice but to seek */
+				ret = g_mime_stream_seek (current->stream, real, GMIME_STREAM_SEEK_SET);
+			} else if (real == current->stream->bound_start) {
+				/* special-case: reset() allows all streams (seekable or not) to reset
+				 * their state and positions to bound_start */
+				ret = g_mime_stream_reset (current->stream);
+			} else {
+				ret = -1;
+			}
+		} else {
+			/* no seek required */
+			ret = 0;
+		}
+		
 		if (ret != -1) {
 			stream->position = offset;
 			cat->current = current;
@@ -457,7 +470,7 @@ g_mime_stream_cat_add_source (GMimeStreamCat *cat, GMimeStream *source)
 	node->next = NULL;
 	node->stream = source;
 	node->length = len;
-	g_mime_stream_ref (source);
+	g_object_ref (source);
 	
 	p = cat->sources;
 	while (p && p->next)
