@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -32,7 +29,7 @@ static void g_mime_stream_file_init (GMimeStreamFile *stream, GMimeStreamFileCla
 static void g_mime_stream_file_finalize (GObject *object);
 
 static ssize_t stream_read (GMimeStream *stream, char *buf, size_t len);
-static ssize_t stream_write (GMimeStream *stream, char *buf, size_t len);
+static ssize_t stream_write (GMimeStream *stream, const char *buf, size_t len);
 static int stream_flush (GMimeStream *stream);
 static int stream_close (GMimeStream *stream);
 static gboolean stream_eos (GMimeStream *stream);
@@ -136,7 +133,7 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 }
 
 static ssize_t
-stream_write (GMimeStream *stream, char *buf, size_t len)
+stream_write (GMimeStream *stream, const char *buf, size_t len)
 {
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
 	ssize_t nwritten;
@@ -304,6 +301,9 @@ stream_substream (GMimeStream *stream, off_t start, off_t end)
  *
  * Creates a new GMimeStreamFile object around @fp.
  *
+ * Note: The created GMimeStreamFile object will own the FILE pointer
+ * passed in.
+ *
  * Returns a stream using @fp.
  **/
 GMimeStream *
@@ -330,6 +330,9 @@ g_mime_stream_file_new (FILE *fp)
  * Creates a new GMimeStreamFile object around @fp with bounds @start
  * and @end.
  *
+ * Note: The created GMimeStreamFile object will own the FILE pointer
+ * passed in.
+ *
  * Returns a stream using @fp with bounds @start and @end.
  **/
 GMimeStream *
@@ -345,3 +348,40 @@ g_mime_stream_file_new_with_bounds (FILE *fp, off_t start, off_t end)
 	
 	return GMIME_STREAM (fstream);
 }
+
+
+/**
+ * g_mime_stream_file_get_owner:
+ * @stream: file stream
+ *
+ * Gets whether or not @stream owns the backend FILE pointer.
+ *
+ * Returns %TRUE if @stream owns the backend FILE pointer or %FALSE
+ * otherwise.
+ **/
+gboolean
+g_mime_stream_file_get_owner (GMimeStreamFile *stream)
+{
+	g_return_val_if_fail (GMIME_IS_STREAM_FILE (stream), FALSE);
+	
+	return stream->owner;
+}
+
+
+/**
+ * g_mime_stream_file_set_owner:
+ * @stream: file stream
+ * @owner: owner
+ *
+ * Sets whether or not @stream owns the backend FILE pointer.
+ *
+ * Note: @owner should be %TRUE if the stream should fclose() the
+ * backend FILE pointer when destroyed or %FALSE otherwise.
+ **/
+void
+g_mime_stream_file_set_owner (GMimeStreamFile *stream, gboolean owner)
+{
+	g_return_if_fail (GMIME_IS_STREAM_FILE (stream));
+	
+	stream->owner = owner;
+}
