@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -34,7 +31,7 @@ static void g_mime_stream_null_init (GMimeStreamNull *stream, GMimeStreamNullCla
 static void g_mime_stream_null_finalize (GObject *object);
 
 static ssize_t stream_read (GMimeStream *stream, char *buf, size_t len);
-static ssize_t stream_write (GMimeStream *stream, char *buf, size_t len);
+static ssize_t stream_write (GMimeStream *stream, const char *buf, size_t len);
 static int stream_flush (GMimeStream *stream);
 static int stream_close (GMimeStream *stream);
 static gboolean stream_eos (GMimeStream *stream);
@@ -99,6 +96,7 @@ static void
 g_mime_stream_null_init (GMimeStreamNull *stream, GMimeStreamNullClass *klass)
 {
 	stream->written = 0;
+	stream->newlines = 0;
 }
 
 static void
@@ -119,9 +117,17 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 }
 
 static ssize_t
-stream_write (GMimeStream *stream, char *buf, size_t len)
+stream_write (GMimeStream *stream, const char *buf, size_t len)
 {
 	GMimeStreamNull *null = (GMimeStreamNull *) stream;
+	register const char *inptr = buf;
+	const char *inend = buf + len;
+	
+	while (inptr < inend) {
+		if (*inptr == '\n')
+			null->newlines++;
+		inptr++;
+	}
 	
 	null->written += len;
 	stream->position += len;
@@ -153,6 +159,7 @@ stream_reset (GMimeStream *stream)
 	GMimeStreamNull *null = (GMimeStreamNull *) stream;
 	
 	null->written = 0;
+	null->newlines = 0;
 	
 	return 0;
 }
