@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,37 +15,33 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
 #ifndef __GMIME_STREAM_H__
 #define __GMIME_STREAM_H__
 
-#ifdef __cplusplus
-extern "C" {
-#pragma }
-#endif /* __cplusplus */
-
 #include <glib.h>
 #include <glib-object.h>
+
+#include <stdio.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <stdarg.h>
 
-#include <gmime/gmime-type-utils.h>
+G_BEGIN_DECLS
 
 #define GMIME_TYPE_STREAM            (g_mime_stream_get_type ())
-#define GMIME_STREAM(obj)            (GMIME_CHECK_CAST ((obj), GMIME_TYPE_STREAM, GMimeStream))
-#define GMIME_STREAM_CLASS(klass)    (GMIME_CHECK_CLASS_CAST ((klass), GMIME_TYPE_STREAM, GMimeStreamClass))
-#define GMIME_IS_STREAM(obj)         (GMIME_CHECK_TYPE ((obj), GMIME_TYPE_STREAM))
-#define GMIME_IS_STREAM_CLASS(klass) (GMIME_CHECK_CLASS_TYPE ((klass), GMIME_TYPE_STREAM))
-#define GMIME_STREAM_GET_CLASS(obj)  (GMIME_CHECK_GET_CLASS ((obj), GMIME_TYPE_STREAM, GMimeStreamClass))
+#define GMIME_STREAM(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GMIME_TYPE_STREAM, GMimeStream))
+#define GMIME_STREAM_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GMIME_TYPE_STREAM, GMimeStreamClass))
+#define GMIME_IS_STREAM(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GMIME_TYPE_STREAM))
+#define GMIME_IS_STREAM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GMIME_TYPE_STREAM))
+#define GMIME_STREAM_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GMIME_TYPE_STREAM, GMimeStreamClass))
 
 typedef enum {
 	GMIME_STREAM_SEEK_SET = SEEK_SET,
 	GMIME_STREAM_SEEK_CUR = SEEK_CUR,
-	GMIME_STREAM_SEEK_END = SEEK_END,
+	GMIME_STREAM_SEEK_END = SEEK_END
 } GMimeSeekWhence;
 
 typedef struct {
@@ -72,16 +66,16 @@ struct _GMimeStream {
 struct _GMimeStreamClass {
 	GObjectClass parent_class;
 	
-	ssize_t  (*read)    (GMimeStream *stream, char *buf, size_t len);
-	ssize_t  (*write)   (GMimeStream *stream, char *buf, size_t len);
-	int      (*flush)   (GMimeStream *stream);
-	int      (*close)   (GMimeStream *stream);
-	gboolean (*eos)     (GMimeStream *stream);
-	int      (*reset)   (GMimeStream *stream);
-	off_t    (*seek)    (GMimeStream *stream, off_t offset, GMimeSeekWhence whence);
-	off_t    (*tell)    (GMimeStream *stream);
-	ssize_t  (*length)  (GMimeStream *stream);
-	GMimeStream *(*substream) (GMimeStream *stream, off_t start, off_t end);
+	ssize_t  (* read)   (GMimeStream *stream, char *buf, size_t len);
+	ssize_t  (* write)  (GMimeStream *stream, const char *buf, size_t len);
+	int      (* flush)  (GMimeStream *stream);
+	int      (* close)  (GMimeStream *stream);
+	gboolean (* eos)    (GMimeStream *stream);
+	int      (* reset)  (GMimeStream *stream);
+	off_t    (* seek)   (GMimeStream *stream, off_t offset, GMimeSeekWhence whence);
+	off_t    (* tell)   (GMimeStream *stream);
+	ssize_t  (* length) (GMimeStream *stream);
+	GMimeStream * (* substream) (GMimeStream *stream, off_t start, off_t end);
 };
 
 
@@ -92,7 +86,7 @@ void g_mime_stream_construct (GMimeStream *stream, off_t start, off_t end);
 
 /* public methods */
 ssize_t   g_mime_stream_read    (GMimeStream *stream, char *buf, size_t len);
-ssize_t   g_mime_stream_write   (GMimeStream *stream, char *buf, size_t len);
+ssize_t   g_mime_stream_write   (GMimeStream *stream, const char *buf, size_t len);
 int       g_mime_stream_flush   (GMimeStream *stream);
 int       g_mime_stream_close   (GMimeStream *stream);
 gboolean  g_mime_stream_eos     (GMimeStream *stream);
@@ -103,8 +97,10 @@ ssize_t   g_mime_stream_length  (GMimeStream *stream);
 
 GMimeStream *g_mime_stream_substream (GMimeStream *stream, off_t start, off_t end);
 
+#ifndef GMIME_DISABLE_DEPRECATED
 void      g_mime_stream_ref     (GMimeStream *stream);
 void      g_mime_stream_unref   (GMimeStream *stream);
+#endif
 
 void      g_mime_stream_set_bounds (GMimeStream *stream, off_t start, off_t end);
 
@@ -116,8 +112,6 @@ ssize_t   g_mime_stream_write_to_stream (GMimeStream *src, GMimeStream *dest);
 
 ssize_t    g_mime_stream_writev (GMimeStream *stream, GMimeStreamIOVector *vector, size_t count);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __GMIME_STREAM_H__ */
