@@ -1,9 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Michael Zucchi <notzed@ximian.com>
- *           Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2000-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -218,33 +214,33 @@ g_mime_utils_header_format_date (time_t time, int offset)
 
 struct _date_token {
 	struct _date_token *next;
-	const unsigned char *start;
-	unsigned int len;
-	unsigned int mask;
+	unsigned char mask;
+	const char *start;
+	size_t len;
 };
 
 static struct _date_token *
 datetok (const char *date)
 {
 	struct _date_token *tokens = NULL, *token, *tail = (struct _date_token *) &tokens;
-	const unsigned char *start, *end;
-	unsigned int mask;
+	const char *start, *end;
+        unsigned char mask;
 	
 	start = date;
 	while (*start) {
 		/* kill leading whitespace */
-		while (*start && isspace ((int) *start))
+		while (*start == ' ' || *start == '\t')
 			start++;
 		
 		if (*start == '\0')
 			break;
 		
-		mask = gmime_datetok_table[*start];
+		mask = gmime_datetok_table[(unsigned char) *start];
 		
 		/* find the end of this token */
 		end = start + 1;
 		while (*end && !strchr ("-/,\t\r\n ", *end))
-			mask |= gmime_datetok_table[*end++];
+			mask |= gmime_datetok_table[(unsigned char) *end++];
 		
 		if (end != start) {
 			token = g_malloc (sizeof (struct _date_token));
@@ -267,11 +263,11 @@ datetok (const char *date)
 }
 
 static int
-decode_int (const unsigned char *in, unsigned int inlen)
+decode_int (const char *in, size_t inlen)
 {
-	register const unsigned char *inptr;
-	const unsigned char *inend;
+	register const char *inptr;
 	int sign = 1, val = 0;
+	const char *inend;
 	
 	inptr = in;
 	inend = in + inlen;
@@ -283,8 +279,8 @@ decode_int (const unsigned char *in, unsigned int inlen)
 		inptr++;
 	
 	for ( ; inptr < inend; inptr++) {
-		if (!isdigit ((int) *inptr))
-			return  -1;
+		if (!(*inptr >= '0' && *inptr <= '9'))
+			return -1;
 		else
 			val = (val * 10) + (*inptr - '0');
 	}
@@ -324,7 +320,7 @@ get_days_in_month (int month, int year)
 #endif
 
 static int
-get_wday (const unsigned char *in, unsigned int inlen)
+get_wday (const char *in, size_t inlen)
 {
 	int wday;
 	
@@ -333,15 +329,16 @@ get_wday (const unsigned char *in, unsigned int inlen)
 	if (inlen < 3)
 		return -1;
 	
-	for (wday = 0; wday < 7; wday++)
-		if (!strncasecmp (in, tm_days[wday], 3))
+	for (wday = 0; wday < 7; wday++) {
+		if (!g_ascii_strncasecmp (in, tm_days[wday], 3))
 			return wday;
+	}
 	
 	return -1;  /* unknown week day */
 }
 
 static int
-get_mday (const unsigned char *in, unsigned int inlen)
+get_mday (const char *in, size_t inlen)
 {
 	int mday;
 	
@@ -356,7 +353,7 @@ get_mday (const unsigned char *in, unsigned int inlen)
 }
 
 static int
-get_month (const unsigned char *in, unsigned int inlen)
+get_month (const char *in, size_t inlen)
 {
 	int i;
 	
@@ -365,22 +362,22 @@ get_month (const unsigned char *in, unsigned int inlen)
 	if (inlen < 3)
 		return -1;
 	
-	for (i = 0; i < 12; i++)
-		if (!strncasecmp (in, tm_months[i], 3))
+	for (i = 0; i < 12; i++) {
+		if (!g_ascii_strncasecmp (in, tm_months[i], 3))
 			return i;
+	}
 	
 	return -1;  /* unknown month */
 }
 
 static int
-get_year (const unsigned char *in, unsigned int inlen)
+get_year (const char *in, size_t inlen)
 {
 	int year;
 	
 	g_return_val_if_fail (in != NULL, -1);
 	
-	year = decode_int (in, inlen);
-	if (year == -1)
+	if ((year = decode_int (in, inlen)) == -1)
 		return -1;
 	
 	if (year < 100)
@@ -393,11 +390,11 @@ get_year (const unsigned char *in, unsigned int inlen)
 }
 
 static gboolean
-get_time (const unsigned char *in, unsigned int inlen, int *hour, int *min, int *sec)
+get_time (const char *in, size_t inlen, int *hour, int *min, int *sec)
 {
-	register const unsigned char *inptr;
-	const unsigned char *inend;
+	register const char *inptr;
 	int *val, colons = 0;
+	const char *inend;
 	
 	*hour = *min = *sec = 0;
 	
@@ -416,7 +413,7 @@ get_time (const unsigned char *in, unsigned int inlen, int *hour, int *min, int
 			default:
 				return FALSE;
 			}
-		} else if (!isdigit ((int) *inptr))
+		} else if (!(*inptr >= '0' && *inptr <= '9'))
 			return FALSE;
 		else
 			*val = (*val * 10) + (*inptr - '0');
@@ -428,8 +425,8 @@ get_time (const unsigned char *in, unsigned int inlen, int *hour, int *min, int
 static int
 get_tzone (struct _date_token **token)
 {
-	const unsigned char *inptr, *inend;
-	unsigned int inlen;
+	const char *inptr, *inend;
+	size_t inlen;
 	int i, t;
 	
 	for (i = 0; *token && i < 2; *token = (*token)->next, i++) {
@@ -449,7 +446,7 @@ get_tzone (struct _date_token **token)
 			}
 			
 			for (t = 0; t < 15; t++) {
-				unsigned int len = strlen (tz_offsets[t].name);
+				size_t len = strlen (tz_offsets[t].name);
 				
 				if (len != inlen)
 					continue;
@@ -590,7 +587,7 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 				d(printf ("weekday; "));
 				got_wday = TRUE;
 				tm.tm_wday = n;
-				goto next_token;
+				goto next;
 			}
 		}
 		
@@ -599,7 +596,7 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 				d(printf ("month; "));
 				got_month = TRUE;
 				tm.tm_mon = n;
-				goto next_token;
+				goto next;
 			}
 		}
 		
@@ -609,7 +606,7 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 				tm.tm_hour = hour;
 				tm.tm_min = min;
 				tm.tm_sec = sec;
-				goto next_token;
+				goto next;
 			}
 		}
 		
@@ -620,7 +617,7 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 				d(printf ("tzone; "));
 				got_tzone = TRUE;
 				offset = n;
-				goto next_token;
+				goto next;
 			}
 		}
 		
@@ -629,37 +626,43 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 				if ((n = get_year (token->start, token->len)) != -1) {
 					d(printf ("year; "));
 					tm.tm_year = n - 1900;
-					goto next_token;
+					goto next;
 				}
 			} else {
-				if (!got_month && !got_wday && token->next && is_numeric (token->next)) {
-					d(printf ("mon; "));
-					n = decode_int (token->start, token->len);
-					got_month = TRUE;
-					tm.tm_mon = n - 1;
-					goto next_token;
+				/* Note: assumes MM-DD-YY ordering if '0 < MM < 12' holds true */
+				if (!got_month && token->next && is_numeric (token->next)) {
+					if ((n = decode_int (token->start, token->len)) > 12) {
+						goto mday;
+					} else if (n > 0) {
+						d(printf ("mon; "));
+						got_month = TRUE;
+						tm.tm_mon = n - 1;
+					}
+					goto next;
 				} else if (!tm.tm_mday && (n = get_mday (token->start, token->len)) != -1) {
+				mday:
 					d(printf ("mday; "));
 					tm.tm_mday = n;
-					goto next_token;
+					goto next;
 				} else if (!tm.tm_year) {
-					d(printf ("2-digit year; "));
-					n = get_year (token->start, token->len);
-					tm.tm_year = n - 1900;
-					goto next_token;
+					if ((n = get_year (token->start, token->len)) != -1) {
+						printf ("2-digit year; ");
+						tm.tm_year = n - 1900;
+					}
+					goto next;
 				}
 			}
 		}
 		
 		d(printf ("???; "));
 		
-	next_token:
+	next:
 		
 		token = token->next;
 	}
 	
 	d(printf ("\n"));
-		
+	
 	t = mktime_utc (&tm);
 	
 	/* t is now GMT of the time we want, but not offset by the timezone ... */
@@ -675,7 +678,7 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 
 #if 0
 static void
-gmime_datetok_table_init ()
+gmime_datetok_table_init (void)
 {
 	int i;
 	
@@ -736,7 +739,12 @@ g_mime_utils_header_decode_date (const char *in, int *saveoffset)
 	struct _date_token *token, *tokens;
 	time_t date;
 	
-	tokens = datetok (in);
+	if (!(tokens = datetok (in))) {
+		if (saveoffset)
+			*saveoffset = 0;
+		
+		return (time_t) 0;
+	}
 	
 	date = parse_rfc822_date (tokens, saveoffset);
 	if (!date)
@@ -976,6 +984,21 @@ g_mime_references_clear (GMimeReferences **refs)
 }
 
 
+/**
+ * g_mime_references_next:
+ * @ref: a GMimeReferences list
+ *
+ * Advances to the next reference node in the GMimeReferences list.
+ *
+ * Returns the next reference node in the GMimeReferences list.
+ **/
+GMimeReferences *
+g_mime_references_next (const GMimeReferences *ref)
+{
+	return ref ? ref->next : NULL;
+}
+
+
 static gboolean
 is_rfc2047_token (const char *inptr, size_t len)
 {
@@ -1070,7 +1093,8 @@ header_fold (const char *in, gboolean structured)
 				   mailer folded it */
 				g_string_append (out, "\n\t");
 				outlen = 1;
-				inptr++;
+				while (isblank (*inptr))
+					inptr++;
 			} else {
 				g_string_append_c (out, *inptr++);
 				outlen++;
@@ -1423,7 +1447,7 @@ rfc2047_decode_word (const unsigned char *in, size_t inlen)
 			*p = '\0';
 		
 		/* slight optimization */
-		if (!strcasecmp (charset, "UTF-8"))
+		if (!g_ascii_strcasecmp (charset, "UTF-8"))
 			return g_strndup (decoded, declen);
 		
 		cd = g_mime_iconv_open ("UTF-8", charset);
@@ -1727,7 +1751,7 @@ rfc2047_encode_word (GString *string, const unsigned char *word, size_t len,
 	int save = 0;
 	char encoding;
 	
-	if (strcasecmp (charset, "UTF-8") != 0)
+	if (g_ascii_strcasecmp (charset, "UTF-8") != 0)
 		cd = g_mime_iconv_open (charset, "UTF-8");
 	
 	if (cd != (iconv_t) -1) {
@@ -2088,7 +2112,7 @@ g_mime_utils_header_encode_text (const unsigned char *in)
  * Decodes an rfc2047 encoded header.
  *
  * WARNING: This function is deprecated. Use
- * g_mime_utils_header_decode_text() instead.
+ * #g_mime_utils_header_decode_text instead.
  *
  * Returns the decoded header (which will be in UTF-8 if at all
  * possible).
@@ -2107,7 +2131,7 @@ g_mime_utils_8bit_header_decode (const unsigned char *in)
  * Encodes a 'text' header according to the rules in rfc2047.
  *
  * WARNING: This function is deprecated. Use
- * g_mime_utils_header_encode_text() instead.
+ * #g_mime_utils_header_encode_text instead.
  *
  * Returns the encoded header. Useful for encoding
  * headers like "Subject".
@@ -2126,7 +2150,7 @@ g_mime_utils_8bit_header_encode (const unsigned char *in)
  * Encodes a 'phrase' header according to the rules in rfc2047.
  *
  * WARNING: This function is deprecated. Use
- * g_mime_utils_header_encode_phrase() instead.
+ * #g_mime_utils_header_encode_phrase instead.
  *
  * Returns the encoded 'phrase'. Useful for encoding internet
  * addresses.
@@ -2147,7 +2171,7 @@ g_mime_utils_8bit_header_encode_phrase (const unsigned char *in)
  * @save: leftover bits that have not yet been encoded
  *
  * Base64 encodes the input stream to the output stream. Call this
- * when finished encoding data with g_mime_utils_base64_encode_step()
+ * when finished encoding data with #g_mime_utils_base64_encode_step
  * to flush off the last little bit.
  *
  * Returns the number of bytes encoded.
@@ -2348,7 +2372,7 @@ g_mime_utils_base64_decode_step (const unsigned char *in, size_t inlen, unsigned
  * @save: leftover bits that have not yet been encoded
  *
  * Uuencodes a chunk of data. Call this when finished encoding data
- * with g_mime_utils_uuencode_step() to flush off the last little bit.
+ * with #g_mime_utils_uuencode_step to flush off the last little bit.
  *
  * Returns the number of bytes encoded.
  **/
@@ -2609,7 +2633,7 @@ g_mime_utils_uudecode_step (const unsigned char *in, size_t inlen, unsigned char
  * @save: leftover bits that have not yet been encoded
  *
  * Quoted-printable encodes a block of text. Call this when finished
- * encoding data with g_mime_utils_quoted_encode_step() to flush off
+ * encoding data with #g_mime_utils_quoted_encode_step to flush off
  * the last little bit.
  *
  * Returns the number of bytes encoded.
@@ -2741,7 +2765,7 @@ g_mime_utils_quoted_encode_step (const unsigned char *in, size_t inlen, unsigned
 
 
 /**
- * g_mime_utils_quoted_decode_step: decode a chunk of QP encoded data
+ * g_mime_utils_quoted_decode_step:
  * @in: input stream
  * @inlen: max length of data to decode
  * @out: output stream
