@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2000-2002 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,18 +15,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
 #ifndef __GMIME_H__
 #define __GMIME_H__
 
-#ifdef __cplusplus
-extern "C" {
-#pragma }
-#endif /* __cplusplus */
-
 #include <glib.h>
 #include <gmime/gmime-error.h>
 #include <gmime/gmime-charset.h>
@@ -76,18 +68,18 @@ extern "C" {
 #include <gmime/gmime-cipher-context.h>
 #include <gmime/gmime-gpg-context.h>
 
-/* GMIME version */
-static const guint gmime_major_version = 2;
-static const guint gmime_minor_version = 2;
-static const guint gmime_micro_version = 1;
-static const guint gmime_interface_age = 0;
-static const guint gmime_binary_age    = 0;
+G_BEGIN_DECLS
 
-#define GMIME_CHECK_VERSION(major,minor,micro)    \
-    (2 > (major) || \
-     (2 == (major) && 2 > (minor)) || \
-     (2 == (major) && 2 == (minor) && \
-      1 >= (micro)))
+/* GMIME version */
+extern const guint gmime_major_version;
+extern const guint gmime_minor_version;
+extern const guint gmime_micro_version;
+extern const guint gmime_interface_age;
+extern const guint gmime_binary_age;
+
+#define GMIME_CHECK_VERSION(major,minor,micro) g_mime_check_version (major, minor, micro)
+
+gboolean g_mime_check_version (guint major, guint minor, guint micro);
 
 
 /**
@@ -104,8 +96,6 @@ static const guint gmime_binary_age    = 0;
 void g_mime_init (guint32 flags);
 void g_mime_shutdown (void);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __GMIME_H__ */
