@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2002 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 1999-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -26,6 +23,7 @@
 #endif
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -73,7 +71,7 @@ uufopen (const char *filename, const char *rw, int flags, mode_t mode)
 		if ((fd = open (filename, flags, mode)) == -1)
 			return NULL;
 	} else {
-		fd = flags & O_RDONLY == O_RDONLY ? 0 : 1;
+		fd = (flags & O_RDONLY) == O_RDONLY ? 0 : 1;
 		if ((fd = dup (fd)) == -1)
 			return NULL;
 	}
@@ -157,7 +155,7 @@ uudecode (const char *progname, int argc, char **argv)
 		}
 		
 		/* decode the mode */
-		if ((mode = strtoul (p, &q, 8) & 0777) && *q != ' ')
+		if ((mode = strtoul (p, (char **) &q, 8) & 0777) && *q != ' ')
 			goto nexti;
 		
 		while (*q == ' ')
