@@ -31,8 +31,10 @@
 
 #include <gmime/gmime.h>
 
+#ifndef G_OS_WIN32
 #define ENABLE_ZENTIMER
 #include "zentimer.h"
+#endif
 
 #if 1
 static char *charsets[] = {
@@ -197,12 +199,14 @@ int main (int argc, char **argv)
 {
 	g_mime_iconv_init ();
 	
-	ZenTimerStart();
+	ZenTimerStart (NULL);
 	test_cache ();
-	ZenTimerStop();
-	ZenTimerReport("test_cache()");
+	ZenTimerStop (NULL);
+	ZenTimerReport (NULL, "test_cache()");
 	
 	test_utils ();
 	
+	g_mime_iconv_shutdown ();
+	
 	return 0;
 }
