@@ -25,12 +25,14 @@
 
 #include <gmime/gmime.h>
 
+#ifndef G_OS_WIN32
 #define ENABLE_ZENTIMER
 #include "zentimer.h"
+#endif
 
 static iconv_t cd = (iconv_t) -1;
 
-void
+static void
 test_parser (char *data)
 {
 	GMimeParser *parser;
@@ -55,7 +57,7 @@ test_parser (char *data)
 	g_mime_object_unref (GMIME_OBJECT (message));
 }
 
-void
+static void
 test_multipart (void)
 {
 	GMimeMessage *message;
@@ -123,7 +125,7 @@ test_multipart (void)
 	g_free (text);
 }
 
-void
+static void
 test_onepart (void)
 {
 	GMimeMessage *message;
@@ -172,27 +174,28 @@ static char *string = "I have no idea what to test here so I'll just test a "
 "foo   \n"
 "Lets also try some tabs in here like this:\t\tis that 2 tabs? I hope so.";
 
-void
+static void
 test_encodings (void)
 {
 	int pos, state = -1, save = 0;
-	char *enc, *dec;
+	char *enc, *dec, *koi8r;
+	iconv_t ic;
 	
 	enc = "=?iso-8859-1?Q?Copy_of_Rapport_fra_Norges_R=E5fisklag.doc?=";
 	fprintf (stderr, "encoded: %s\n", enc);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (dec);
 	
 	enc = "=?iso-8859-1?Q?Copy_of_Rapport_fra_Norges_R=E5fisklagdoc?=";
 	fprintf (stderr, "encoded: %s\n", enc);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (dec);
 	
 	enc = "=?iso-8859-1?B?dGVzdOb45S50eHQ=?=";
 	fprintf (stderr, "encoded: %s\n", enc);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (dec);
 	
@@ -200,99 +203,99 @@ test_encodings (void)
 		"=?windows-1250?Q?=2C_p=F8edloh_do_A4=2C_=E8/b_lasertov=FD_ti?= "
 		"=?windows-1250?Q?sk_a_=E8/b_inkoutov=FD_tisk_do_A2!!!?=";
 	fprintf (stderr, "encoded: %s\n", enc);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (dec);
 	
 	enc = "OT - ich =?ISO-8859-1?Q?wei=DF?=, trotzdem";
 	fprintf (stderr, "encoded: %s\n", enc);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (dec);
 	
 	enc = "OT - ich =?iso-8859-1?b?d2Vp3yw=?= trotzdem";
 	fprintf (stderr, "encoded: %s\n", enc);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (dec);
 	
 	enc = "OT - ich =?ISO-8859-1?Q?wei=DF,?= trotzdem";
 	fprintf (stderr, "encoded: %s\n", enc);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (dec);
 	
 	dec = g_mime_iconv_strdup (cd, "OT - ich wei, trotzdem");
 	fprintf (stderr, "pre-encoded: %s\n", dec);
-	enc = g_mime_utils_8bit_header_encode (dec);
+	enc = g_mime_utils_header_encode_text (dec);
 	g_free (dec);
 	fprintf (stderr, "encoded: %s\n", enc);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (enc);
 	g_free (dec);
 	
 	enc = g_strdup ("=?iso-8859-1?Q?=DE?=:\t=?iso-8859-1?Q?=AD=BE=EF?= spells 0xdeadbeef");
 	fprintf (stderr, "incorrect: %s\n", enc);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (enc);
-	enc = g_mime_utils_8bit_header_encode (dec);
+	enc = g_mime_utils_header_encode_text (dec);
 	g_free (dec);
 	fprintf (stderr, "correct: %s\n", enc);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (enc);
 	g_free (dec);
 	
 	enc = g_strdup ("=?iso-8859-1?q?blablah?=");
 	fprintf (stderr, "encoded: %s\n", enc);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (enc);
 	g_free (dec);
 	
 	enc = g_strdup ("=?iso-8859-1?Q?blablah?=");
 	fprintf (stderr, "encoded: %s\n", enc);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (enc);
 	g_free (dec);
 	
 	dec = g_mime_iconv_strdup (cd, "Kristoffer Brnemyr");
 	fprintf (stderr, "pre-encoded: %s\n", dec);
-	enc = g_mime_utils_8bit_header_encode (dec);
+	enc = g_mime_utils_header_encode_phrase (dec);
 	g_free (dec);
 	fprintf (stderr, "encoded: %s\n", enc);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_phrase (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (enc);
 	g_free (dec);
 	
 	dec = g_mime_iconv_strdup (cd, "aa aaaa aa");
-	enc = g_mime_utils_8bit_header_encode (dec);
+	enc = g_mime_utils_header_encode_text (dec);
 	fprintf (stderr, "encoded: %s\n", enc);
 	g_free (dec);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (enc);
 	g_free (dec);
 	
 	dec = g_mime_iconv_strdup (cd, "aa aaaa aa");
-	enc = g_mime_utils_8bit_header_encode_phrase (dec);
+	enc = g_mime_utils_header_encode_phrase (dec);
 	fprintf (stderr, "encoded: %s\n", enc);
 	g_free (dec);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_phrase (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (enc);
 	g_free (dec);
 	
 	fprintf (stderr, "test that white space between 8bit words is preserved\n");
 	dec = g_mime_iconv_strdup (cd, "aa  \t  aa");
-	enc = g_mime_utils_8bit_header_encode (dec);
+	enc = g_mime_utils_header_encode_text (dec);
 	fprintf (stderr, "encoded: %s\n", enc);
 	g_free (dec);
-	dec = g_mime_utils_8bit_header_decode (enc);
+	dec = g_mime_utils_header_decode_text (enc);
 	fprintf (stderr, "decoded: %s\n", dec);
 	g_free (enc);
 	g_free (dec);
@@ -309,6 +312,24 @@ test_encodings (void)
 	fprintf (stderr, "unquoted: %s\n", enc);
 	g_free (enc);
 	
+	enc = "=?iso-8859-5?b?tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2tra2trY=?=";
+	koi8r = g_mime_utils_header_decode_text (enc);
+	
+	
+	enc = g_mime_utils_header_encode_text (koi8r);
+	fprintf (stderr, "encode_text: %s\n", enc);
+	dec = g_mime_utils_header_decode_text (enc);
+	fprintf (stderr, "decoded (%s): %s\n", !strcmp (koi8r, dec) ? "GOOD" : "BAD", dec);
+	g_free (enc);
+	g_free (dec);
+	
+	enc = g_mime_utils_header_encode_phrase (koi8r);
+	fprintf (stderr, "encode_phrase: %s\n", enc);
+	dec = g_mime_utils_header_decode_phrase (enc);
+	fprintf (stderr, "decoded (%s): %s\n", !strcmp (koi8r, dec) ? "GOOD" : "BAD", dec);
+	g_free (enc);
+	g_free (dec);
+	
 	enc = g_malloc (strlen (string) * 3);
 	pos = g_mime_utils_quoted_encode_close (string, strlen (string), enc, &state, &save);
 	enc[pos] = '\0';
@@ -343,8 +364,11 @@ static char *addresses[] = {
 	"Tivo Leedjrv <leedjarv@interest.ee>",
 	"fbosi@mokabyte.it;, rspazzoli@mokabyte.it",
 	"\"Miles (Star Trekkin) O'Brian\" <mobrian@starfleet.org>",
-	"Patrick Lamaizire <email@domain>",
-	"j(effrey).stedfast@novell.com",
+	"undisclosed-recipients: ;",
+	"undisclosed-recipients:;",
+	"undisclosed-recipients:",
+	"undisclosed-recipients;",
+	"undisclosed-recipients",
 	NULL
 };
 
@@ -367,8 +391,7 @@ dump_addrlist (InternetAddressList *addrlist, int i, gboolean group, gboolean de
 		}
 		
 		if (ia->type == INTERNET_ADDRESS_GROUP) {
-			fprintf (stderr, "Address is a group:\n");
-			fprintf (stderr, "Name: %s\n", ia->name ? ia->name : "");
+			fprintf (stderr, "Group Name: %s\n", ia->name ? ia->name : "");
 			dump_addrlist (ia->value.members, -1, TRUE, FALSE);
 			fprintf (stderr, "End of group.\n");
 		} else if (ia->type == INTERNET_ADDRESS_NAME) {
@@ -390,7 +413,7 @@ dump_addrlist (InternetAddressList *addrlist, int i, gboolean group, gboolean de
 	}
 }
 
-void
+static void
 test_addresses (void)
 {
 	int i;
@@ -412,7 +435,7 @@ test_addresses (void)
 
 /*#include "date-strings.h"*/
 
-void
+static void
 test_date (void)
 {
 	int offset = 0;
@@ -432,7 +455,14 @@ test_date (void)
 	out = g_mime_utils_header_format_date (date, offset);
 	fprintf (stderr, "date out: [%s]\n", out);
 	g_free (out);
-
+	
+	in = "Tue, 30 Mar 2004 13:01:38 +0000";
+	fprintf (stderr, "date  in: [%s]\n", in);
+	date = g_mime_utils_header_decode_date (in, &offset);
+	out = g_mime_utils_header_format_date (date, offset);
+	fprintf (stderr, "date out: [%s]\n", out);
+	g_free (out);
+	
 #if 0
 	{
 		char *string;
@@ -475,5 +505,7 @@ int main (int argc, char *argv[])
 	
 	g_mime_iconv_close (cd);
 	
+	g_mime_shutdown ();
+	
 	return 0;
 }
